/***************************************************************************
 *   Copyright (C) 2007 by Andrea Moro   *
 *   andrea_moro@email.it   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 ***************************************************************************/
#ifndef ADFGVX_H
#define ADFGVX_H

#include "ui_ADFGVX.h"

class mainWindow;

/** 
 * @short User interface and algorithms for AFGVX cipher.
 *
 * The AFGVX cipher. This class provides the cryptographic algorithms
 * and user interface handling.
 */
class ADFGVX :
		public QWidget,
		public Ui::ADFGVX
{
	Q_OBJECT
public:
	/**
	 * Creates a new ADGVX instance.
	 *
	 * @param *main the main window of CrypTooLinux
	 * @param text the text to encrypt or decrypt
	 * @param title the title of the window that contain the text to encrypt or decrypt
	 */
	ADFGVX(mainWindow *main,QString text, QString title);
private slots:
	/**
	 * Update correctly the check status of the output options.
	 * Called when an option changed check status.
	 */
	void upCheck();
	/**
	 * Update transpositionLine, encryptButton and decryptButton.
	 * Called when the password is changed.
	 */
	void upButton();
	/**
	 * Check if the character inserted is not present in the matrix and if it is
	 * present ask to the user what to do.
	 * Called when an element of the matrix is changed.
	 */
	void upMatr();
	/**
	 * Generate a random valid matrix.
	 */
	void rndMatr();
	/**
	 * Input the matrix just like a string. 
	 */
	void strMatr();
	/**
	 * Clear the matrix.
	 */
	void eraseMatr();
	/**
	 * Generate the standard matrix for ADFGVX.
	 */
	void stdMatr();
	/**
	 * Decrypt the workText with the matrix and password inserted, close the
	 * AFGVX window and create a new window in CrypTooLinux main window with
	 * the decrypted text.
	 */
	void decrypt();
	/**
	 * Encrypt the workText with the matrix and password inserted, close the
	 * AFGVX window and create a new window in CrypTooLinux main window with
	 * the encrypted text.
	 */
	void encrypt();
private:
	/**
	 * Makes the connections between signal of the ui and the slot functions.
	 */
	void createConnections();
	/**
	 * The text to encrypt or decrypt.
	 */
	QString workText;
	/**
	 * The title of window in CrypTooLinux main window that contain the text
	 * to encrypt or decrypt.
	 */
	QString workTitle;
	/**
	 * CrypTooLinux main window.
	 */
	mainWindow *mw;
	/**
	 * The matrix in the user interface.
	 */
	QLineEdit *cell[6][6];
};
#endif
