/***************************************************************************
 *   Copyright (C) 2007 by apterium   *
 *   apterium@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <QtGui>
#include <gmp.h>
#include "CaesarAnalysis.h"
#include "mainWindow.h"
#include "Text.h"
#include "./libClassicalAnalysis/libClassicalAnalysis.h"

CaesarAnalysis::CaesarAnalysis(mainWindow *main, QString text, QString title)
{
	this->workText = text;
	this->workTitle = title;	
	this->mw = main; 
	if (mw->options.text_opt.keep_case) {
		this->cs = Qt::CaseInsensitive;
	}
	else {
		this->cs = Qt::CaseSensitive;
	}
	
	computeFrequencies();
	
	createConnections();
}

void CaesarAnalysis::createConnections()
{
	
	
}

void CaesarAnalysis::computeFrequencies()
{
	//first try with ASCII Art :-)
	QString alphabet = mw->options.text_opt.alphabet;
	
	Text theText(workText, &alphabet, this->cs);
	Text theAlphabet(alphabet, &alphabet, this->cs);
	
	const std::vector<mpz_class> textNbr = theText.getIntRepr();
	const std::vector<mpz_class> alphNbr = theAlphabet.getIntRepr();

	libClassicalAnalysis::Frequencies engine (textNbr, alphNbr);
	std::vector<mpz_class> freq = engine.getSingleFrequencies();
	
	QString result;
	result.append("Frequencies : \n");
	
	for (std::vector<mpz_class>::size_type i = 0 ; i < alphNbr.size() ; i++) {
		result.append(alphabet.at(i));
		result.append(" : ");
		char* freqStr = mpz_get_str(NULL, 10, freq.at(i).get_mpz_t());
		result.append(freqStr);
		result.append("\n");
		delete freqStr;
	}
	
	QString resultTitle = "Frequency analysis of ";
	resultTitle.append(this->workTitle);
	mw->newSubWindow(resultTitle, result);
	
}

//#include "CaesarAnalysis.moc"
