/***************************************************************************
 *   Copyright (C) 2007 by apterium   *
 *   apterium@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <QtGui>
#include "Factorisation.h"
#include "mainWindow.h"
#include "./libNumberTheory/libNumberTheory.h"


Factorisation::Factorisation(const mainWindow *main)
{
	setupUi(this);
	createConnections();
	this->algos = new std::list<libAnalysis::FactorisationAlgorithmIndex>();
	initializeDialog();
	this->theNumber = new mpz_class(0);
	this->theFactors = NULL;
}

Factorisation::~Factorisation()
{
	delete this->theNumber;
	delete this->theFactors;
	delete this->algos;
}

void Factorisation::initializeDialog()
{
	QRegExp re("[0-9]*");
	//parent = this so no need to delete after use.
	QRegExpValidator *v = new QRegExpValidator(re, this);
	number->setValidator(v);
	number->setText("");
	number->setEnabled(true);
	runButton->setEnabled(false);
}

void Factorisation::createConnections()
{
	connect(bruteforce, SIGNAL(stateChanged(int)), this, SLOT(updateAlgorithms()));
	connect(fermat, SIGNAL(stateChanged(int)), this, SLOT(updateAlgorithms()));
	
	connect(runButton, SIGNAL(clicked()), this, SLOT(factorise()));
	connect(number, SIGNAL(textEdited(const QString &)), this, SLOT(defineNumber()));
}

void Factorisation::defineNumber()
{
	if (number->text().length() != 0) {
		QByteArray nbr = number->text().toAscii();
		*(this->theNumber) = mpz_class(nbr.constData(), 10);
	}
	if (*(this->theNumber) != 0) {
		runButton->setEnabled(true);
	}	
}

void Factorisation::updateAlgorithms()
{
	//ADD OTHER ALGORITHMS HERE WHEN ADDING THEM TO THE UI
	if (bruteforce->checkState() == Qt::Unchecked) {
		this->algos->remove(libAnalysis::BRUTEFORCE);
	}
	if (bruteforce->checkState() == Qt::Checked) {
		this->algos->push_back(libAnalysis::BRUTEFORCE);
	}
	if (fermat->checkState() == Qt::Unchecked) {
		this->algos->remove(libAnalysis::FERMAT);
	}
	if (fermat->checkState() == Qt::Checked) {
		this->algos->push_back(libAnalysis::FERMAT);
	}

}

void Factorisation::factorise()
{
	//NOTE : different behaviour from CrypTool.
	//TODO : (if want to mimic CrypTol, not searching every factors of
	//n, but not only : it would perhaps be better). Share the engine
	//through the whole class. Here it should be tested
	//whether it is NULL or not, created if it is. Delete it if the number
	//is updated (that is, when the user modifies it in the window), or when
	//the algorithms list is updated as well.
	
	if (this->algos->empty() ) {
		QMessageBox::warning(this, tr("Error"), tr("You must select at least one algorithm"));
		return;
	}
	if (this->number->text().length() == 0) {
		QMessageBox::warning(this, tr("Error"), tr("You must enter a number."));
		return;
	}
	libAnalysis::CustomFactorisation engine  = 
		libAnalysis::CustomFactorisation(*(this->theNumber), *(this->algos));
	QString result = "";

	//TODO : initialize a new engine for every non prime factor. Actually,
	//the memory overhead is negligible, as factorisation methods
	//(especially advanced ones) are
	//targeted at finding only ONE non-trivial factor.

	while (engine.isPrime() == 0) {
		mpz_class factor = engine.getNextFactor();
				
		char* str = mpz_get_str(NULL, 10, factor.get_mpz_t());
		result.append(str);
		delete str;
		result.append(" * ");
		this->resultLine->setText(result);
		
		//TODO : hire a designer for the UI :)
		QMessageBox::warning(this, tr(""), tr("Click ok to get a new factor"));
	}	
	char* str = mpz_get_str(NULL, 10, engine.getRemainingQuotient().get_mpz_t());
	result.append(str);
	delete str;
	this->resultLine->setText(result);
}


#include "Factorisation.moc"
