/***************************************************************************
 *   Copyright (C) 2007 by apterium   *
 *   apterium@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef __FACTORISATION_H
#define __FACTORISATION_H

#include <list>
#include <gmpxx.h>
#include "./libAnalysis/libAnalysis.h"
#include "ui_Factorisation.h"


class mainWindow;

/**
 * @short User interface for factoring a number.
 * 
 * This class provides the user interface handling and the call for
 * factorisation methods.
 */
class Factorisation : 
	public QWidget,
	public Ui::Factorisation
{
	Q_OBJECT
public:
	/**
	 * Creates a new Factorisation instance.
	 *
	 * @param *main the main window of CrypTooLinux.
	 */
	Factorisation(const mainWindow *main);

	/**
	 * Destructor.
	 * 
	 */
	~Factorisation();

private:
	/**
	 * The list of allowed algorithms
	 * 
	 */
	std::list<libAnalysis::FactorisationAlgorithmIndex> *algos;
	
	/**
	 * The number to factorise.
	 * 
	 */
	mpz_class *theNumber;
			    
	/**
	 * The computed factors of the list.
	 * 
	 */
	std::list<mpz_class> *theFactors;
			 
	/**
	 * Creates the connections between signal of the use interface and the
	 * slot functions.
	 * 
	 */
	void createConnections();
	
	/**
	 * Initialises the dialog. That is, define the initial values of the
	 * elements, limit the possible value for characters in the texts, etc.
	 * 
	 */ 
	void initializeDialog();
	
private slots:
	/**
	 * Runs the factorisation of the given number, using the
	 * algorithms. Launched by click on the Run button. Gives the factors
	 * one by one.
	 *
	 */
	void factorise();
	
	/**
	 * Update the algorithms when the check boxes are 
	 * 
	 */
	void updateAlgorithms();

	/**
	 * Gets the number entered by the user. Update its value. Enable the run
	 * Button.
	 * 
	 */
	void defineNumber();

};

#endif //__FACTORISATION_H
