/***************************************************************************
 *   Copyright (C) 2007 by apterium   *
 *   apterium@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef __PRIME_NUMBER_TEST_H
#define __PRIME_NUMBER_TEST_H

#include <gmpxx.h>
#include "ui_PrimeNumberTest.h"


class mainWindow;

/**
 * @short User interface for testing the primality of a number.
 * 
 * This class provides the user interface handling and the call for
 * primality tests.
 * @author apterium
 */
class PrimeNumberTest : 
	public QWidget,
	public Ui::PrimeNumberTest
{
	Q_OBJECT
public:
	/**
	 * Creates a new PrimeNumberTest instance.
	 *
	 * @param *main the main window of CrypTooLinux.
	 */
	PrimeNumberTest(const mainWindow *main);

	/**
	 * Destructor.
	 * 
	 */
	~PrimeNumberTest();

private:
	/**
	 * The number to test.
	 * 
	 */
	mpz_class *theNumber;
			    
	/**
	 * Creates the connections between signal of the use interface and the
	 * slot functions.
	 * 
	 */
	void createConnections();
	
	/**
	 * Initialises the dialog. That is, define the initial values of the
	 * elements, limit the possible value for characters in the texts, etc.
	 * 
	 */ 
	void initializeDialog();
	
private slots:
	/**
	 * Runs the primality test of the given number, using the
	 * algorithms. Launched by click on the Run button. 
	 *
	 */
	void primalityTest();
	
	/**
	 * Update the algorithm when the user selects one by clicking on a
	 * radio button.
	 * 
	 */
	void updateAlgorithm();

	/**
	 * Gets the number entered by the user. Update its value. Enable the run
	 * button.
	 * 
	 */
	void defineNumber();

};

#endif //__PRIME_NUMBER_TEST_H
