/*************************************************************************** 
 *   Copyright (C) 2007 by apterium   *                                      
 *   apterium@gmail.com   *                                                  
 *                                                                         * 
 *   This program is free software; you can redistribute it and/or modify  * 
 *   it under the terms of the GNU General Public License as published by  * 
 *   the Free Software Foundation; either version 3 of the License, or     * 
 *   (at your option) any later version.                                   * 
 *                                                                         * 
 *   This program is distributed in the hope that it will be useful,       * 
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        * 
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         * 
 *   GNU General Public License for more details.                          * 
 *                                                                         * 
 *   You should have received a copy of the GNU General Public License     * 
 *   along with this program; if not, write to the                         * 
 *   Free Software Foundation, Inc.,                                       * 
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             * 
 ***************************************************************************/

#include <iostream>
#include <cstdlib>
#include "Text.h"

Text::Text(const QString& strng, const QString* alphabt, const Qt::CaseSensitivity cs)
{
	this->initialString = new QString(strng);
	this->alphabet = alphabt;
	this->updateIntRep();
	this->caseSens = cs;
}

Text::~Text()
{
	delete this->initialString;
	delete this->initIntRep;
}

const std::vector<mpz_class>& Text::getIntRepr() const
{
	return *(this->initIntRep);
}

std::vector<mpz_class>::size_type Text::getWorkTextLength() const
{
	return this->initIntRep->size();
}

void Text::addZero()
{
	this->initialString->append(this->alphabet->at(0));
	this->initIntRep->push_back(mpz_class(0));
}

mpz_class Text::getAlphabetCardinality() const
{
	return mpz_class(this->alphabet->length());
}

void Text::setResultingText(const std::vector<mpz_class>& resText)
{
	#ifdef DEBUG
	std::cout << "Line " << __LINE__ << " of " << __FILE__ << " : " << std::endl;
	std::cout << "Setting the integer representation of a text..." << std::endl;
        #endif
	
	this->resultIntRep = new std::vector<mpz_class>(resText);
	this->updateStringRep();	
}

void Text::updateIntRep()
{
	this->initIntRep = new std::vector<mpz_class>;
	for (int i = 0 ; i < this->initialString->length() ; i++) {
		int charPosition = this->alphabet->indexOf(this->initialString->at(i), 
							   0, this->caseSens);
		if (charPosition != -1) { 
			//the character is in the alphabet
			this->initIntRep
				->push_back(mpz_class(charPosition));
		}
	}
}

void Text::updateStringRep()
{
	this->resultString = new QString(*(this->initialString));
	//the offset needed because of characters which does not appear in the
	//alphabet
	unsigned int offset = 0;
	for (unsigned int i = 0 ; i < this->resultIntRep->size() ; i++ ) {
		int charPosition = this->alphabet->indexOf(this->initialString->at(i + offset), 
							   0, this->caseSens);
		while (charPosition == -1) {
			offset++;
			charPosition = this->alphabet->indexOf(this->initialString->at(i+offset), 
							       0, this->caseSens);
		}
		//then update resultString
		mpz_class current = this->resultIntRep->at(i);
		if (!(current.fits_uint_p()) ) {
			#ifdef DEBUG
			std::cout << "Error : resulting integer does not fit in the alphabet size : ";
			std::cout << std::endl << " Got : " << current << std::endl;
                        #endif
			exit(EXIT_FAILURE);
		}
		unsigned int index = (unsigned int) current.get_si();
		if (this->caseSens == Qt::CaseSensitive) {
			//then directly use the alphabet
			this->resultString->replace(i + offset, 1, 
					    alphabet->at(index));
		}
		else {
			//check the case sensitivity of the charater of the
			//initial text in order to restore the same
			if (this->initialString->at(i + offset).isUpper() ) {
				this->resultString->replace(i + offset, 1, alphabet->at(index).toUpper());
			} 
			else if (this->initialString->at(i + offset).isLower() ) {
				this->resultString->replace(i + offset, 1, alphabet->at(index).toLower());
			}
			else {
				//it is not lower nor upper
				this->resultString->replace(i + offset, 1, alphabet->at(index));
			}
		}
	}
}

