/***************************************************************************
 *   Copyright (C) 2007 by Andrea Moro   *
 *   andrea_moro@email.it   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 ***************************************************************************/
#include <QtGui>
#include "analysisOpt.h"
#include "mainWindow.h"

analysisOpt::analysisOpt(mainWindow *main)
{
	setupUi(this);
	connect(applyButton,SIGNAL(clicked()),this,SLOT(apply()));
	mw=main;

	caesar_derived_key->setCheckState(
		mw->options.analysis_opt.caesar_derived_key?(Qt::Checked):(Qt::Unchecked));
	caesar_correlation->setCheckState(
		mw->options.analysis_opt.caesar_correlation?(Qt::Checked):(Qt::Unchecked));
	caesar_histograms->setCheckState(
		mw->options.analysis_opt.caesar_histograms?(Qt::Checked):(Qt::Unchecked));
	vxa_derived_key->setCheckState(
		mw->options.analysis_opt.vxa_derived_key?(Qt::Checked):(Qt::Unchecked));
	vxa_derived_key_len->setCheckState(
		mw->options.analysis_opt.vxa_derived_key_len?(Qt::Checked):(Qt::Unchecked));
	vxa_autocorrelation->setCheckState(
		mw->options.analysis_opt.vxa_autocorrelation?(Qt::Checked):(Qt::Unchecked));
	vxa_base_analysis->setCheckState(
		mw->options.analysis_opt.vxa_base_analysis?(Qt::Checked):(Qt::Unchecked));

	char tmp[10];
	sprintf(tmp,"%d",mw->options.analysis_opt.num_bytes_entropy);
	num_bytes_entropy->setText(tmp);

	allChar->setChecked(false);
	printChar->setChecked(false);
	selChar->setChecked(false);
	if(mw->options.analysis_opt.cleartext_alph==0)
		allChar->setChecked(true);
	else if(mw->options.analysis_opt.cleartext_alph==1)
		printChar->setChecked(true);
	else
		selChar->setChecked(true);
}

void analysisOpt::apply()
{
	int tmp;
	tmp=atoi(num_bytes_entropy->text().toAscii().data());
	if((tmp<32)||(tmp>4096))
	{
		QMessageBox::warning(this, tr("Warning"),
				     tr("The width can be only in 32-4096!"));
		return;
	}
	mw->options.analysis_opt.num_bytes_entropy=tmp;

	if(allChar->isChecked())
		mw->options.analysis_opt.cleartext_alph=0;
	else if(printChar->isChecked())
		mw->options.analysis_opt.cleartext_alph=1;
	else
		mw->options.analysis_opt.cleartext_alph=2;

	mw->options.analysis_opt.caesar_derived_key=caesar_derived_key->checkState()==Qt::Checked?true:false;
	mw->options.analysis_opt.caesar_correlation=caesar_correlation->checkState()==Qt::Checked?true:false;
	mw->options.analysis_opt.caesar_histograms=caesar_histograms->checkState()==Qt::Checked?true:false;
	mw->options.analysis_opt.vxa_derived_key=vxa_derived_key->checkState()==Qt::Checked?true:false;
	mw->options.analysis_opt.vxa_derived_key_len=vxa_derived_key_len->checkState()==Qt::Checked?true:false;
	mw->options.analysis_opt.vxa_autocorrelation=vxa_autocorrelation->checkState()==Qt::Checked?true:false;
	mw->options.analysis_opt.vxa_base_analysis=vxa_base_analysis->checkState()==Qt::Checked?true:false;
	close();
}

#include "analysisOpt.moc"
