/***************************************************************************
 *   Copyright (C) 2007 by Andrea Moro   *
 *   andrea_moro@email.it   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 ***************************************************************************/
#include <QtGui>
#include "atbash.h"
#include "mdichild.h"
#include "mainWindow.h"

Atbash::Atbash(mainWindow *main,QString text, QString title)
{
	setupUi(this);
	createConnections();
	workText=text;
	workTitle=title;
	mw=main;
}
void Atbash::createConnections()
{
	connect(atbashRadio,SIGNAL(clicked()),this,SLOT(atbashSel()));
	connect(ascRadio,SIGNAL(clicked()),this,SLOT(substSel()));
	connect(descRadio,SIGNAL(clicked()),this,SLOT(substSel()));
	connect(keyLine,SIGNAL(textEdited(const QString &)),this,SLOT(recalculateAlph()));
	connect(ascRadio,SIGNAL(clicked()),this,SLOT(recalculateAlph()));
	connect(descRadio,SIGNAL(clicked()),this,SLOT(recalculateAlph()));
	connect(encryptButton,SIGNAL(clicked()),this,SLOT(encrypt()));
	connect(decryptButton,SIGNAL(clicked()),this,SLOT(decrypt()));
}
void Atbash::substSel()
{
	keyLine->setEnabled(true);
}
void Atbash::atbashSel()
{
	keyLine->setEnabled(false);
	keyLine->setText("ZYXWVUTSRQPONMLKJIHGFEDCBA");
	cryptAlph->setText("ZYXWVUTSRQPONMLKJIHGFEDCBA");
}
void Atbash::recalculateAlph()
{
	keyLine->setText(keyLine->text().toUpper());
	if(keyLine->text().length()!=0)
	{
		QString newAlph;
		QString alph;
		int i,k,j=0;
		if(ascRadio->isChecked())
			alph.insert(0,"ABCDEFGHIJKLMNOPQRSTUVWXYZ");
		else
			alph.insert(0,"ZYXWVUTSRQPONMLKJIHGFEDCBA");
		for(i=0;i<keyLine->text().length();i++)
		{
			bool q=false;
			for(k=0;k<j;k++)
				if(newAlph[k]==keyLine->text()[i])
					q=true;
			if(!q)
				newAlph.append(keyLine->text()[i]);
		}
		i=-1;
		for(;j<27;j++)
		{
			bool q;
			do
			{
				i++;
				q=false;
				for(k=0;k<keyLine->text().length();k++)
				if(alph[i]==keyLine->text()[k])
					q=true;
			}while(q);
			newAlph.append(alph[i]);
		}
		cryptAlph->setText(newAlph);
	}
	else
		cryptAlph->setText("ABCDEFGHIJKLMNOPQRSTUVWXYZ");
}
void Atbash::decrypt()
{
	QString title;
	if(atbashRadio->isChecked())
		title.append(tr("Atbash decryption of <"));
	else
		title.append(tr("Substitution decryption of <"));
	title.append(workTitle);
	title.append(tr(">, key <"));
	title.append(keyLine->text());
	title.append(">");
	QString tmp=realAlph->text(),crypt=cryptAlph->text();
	int len=workText.length(),i;
	for(i=0;i<len;i++)
		if((workText[i]>=QChar('A'))&&(workText[i]<=QChar('Z')))
			workText[i]=tmp[crypt.indexOf(workText[i])];
		else if((workText[i]>=QChar('a'))&&(workText[i]<=QChar('z')))
			workText[i]=tmp[crypt.indexOf(workText[i],0,Qt::CaseInsensitive)].toLower();
	mw->newSubWindow(title,workText);
	close();
}
void Atbash::encrypt()
{
	QString title;
	if(atbashRadio->isChecked())
		title.append(tr("Atbash encryption of <"));
	else
		title.append(tr("Substitution encryption of <"));
	title.append(workTitle);
	title.append(tr(">, key <"));
	title.append(keyLine->text());
	title.append(">");
	QString tmp=cryptAlph->text();
	int len=workText.length(),i;
	for(i=0;i<len;i++)
		if((workText[i]>=QChar('A'))&&(workText[i]<=QChar('Z')))
			workText[i]=tmp[workText[i].toAscii()-'A'];
		else if((workText[i]>=QChar('a'))&&(workText[i]<=QChar('z')))
			workText[i]=tmp[workText[i].toAscii()-'a'].toLower();
	mw->newSubWindow(title,workText);
	close();
}

#include "atbash.moc"
