/***************************************************************************
 *   Copyright (C) 2007 by Andrea Moro   *
 *   andrea_moro@email.it   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 ***************************************************************************/

#ifndef ATBASH_H
#define ATBASH_H

#include "ui_atbash.h"

class mainWindow;

/** 
 * @short User interface and algorithms for Atbash and Substitution ciphers.
 *
 * The Atbash and Substitution ciphers. This class provides the cryptographic algorithms
 * and user interface handling.
 */
class Atbash :
		public QWidget,
		public Ui::Atbash
{
	Q_OBJECT
public:
	/**
	 * Creates a new Atbash instance.
	 *
	 * @param *main the main window of CrypTooLinux
	 * @param text the text to encrypt or decrypt
	 * @param title the title of the window taht contain the text to encrypt or decrypt
	 */
	Atbash(mainWindow *main,QString text, QString title);
private slots:
	/**
	 * Enable the keyline.
	 */
	void substSel();
	/**
	 * Set the key to the key of Atbash cipher.
	 */
	void atbashSel();
	/**
	 * Update the alphabet for the given key.
	 */
	void recalculateAlph();
	/**
	 * Decrypt the workText with the key inserted, close the Atbash window and create
	 * a new window in CrypTooLinux main window with the decrypted text.
	 */
	void decrypt();
	/**
	 * Encrypt the workText with the key inserted, close the Atbash window and create
	 * a new window in CrypTooLinux main window with the encrypted text.
	 */
	void encrypt();
private:
	/**
	 * Makes the connections between signal of the ui and the slot functions.
	 */
	void createConnections();
	/**
	 * The text to encrypt or decrypt.
	 */
	QString workText;
	/**
	 * The title of window in CrypTooLinux main window that contain the text
	 * to encrypt or decrypt.
	 */
	QString workTitle;
	/**
	 * CrypTooLinux main window.
	 */
	mainWindow *mw;
};
#endif
