/***************************************************************************
 *   Copyright (C) 2007 by Andrea Moro   *
 *   andrea_moro@email.it   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 ***************************************************************************/
#include <QtGui>
#include "byteXor.h"
#include "mdichild.h"
#include "mainWindow.h"

byteXor::byteXor(mainWindow *main,QString text, QString title)
{
	setupUi(this);
	createConnections();
	workText=text;
	workTitle=title;
	mw=main;
	kLen=0;
}
void byteXor::adjustKey()
{
	int i;
	for(i=0;i<keyLine->text().length();i++)
		if(((!keyLine->text()[i].isLetterOrNumber())||
			(keyLine->text().toUpper()[i]>'F'))&&
			(keyLine->text()[i]!=' '))
		{
			QMessageBox::warning(this, tr("Warning"),
			tr("The key must contain only these characters 0..9A..F"));
			keyLine->undo();
			return;
		}
	keyLine->setText((keyLine->text().remove(' ')).toUpper());
	kLen=keyLine->text().length()/2+1;
	for(i=1;i<kLen;i++)
		keyLine->setText(keyLine->text().insert((i*3)-1,QChar(' ')));
	if(keyLine->text().endsWith(' '))
		keyLine->setText(keyLine->text().remove(keyLine->text().length()-1,1));
	if(keyLine->text().length()==0)
	{
		encryptButton->setEnabled(false);
		decryptButton->setEnabled(false);
	}
	else
	{
		encryptButton->setEnabled(true);
		decryptButton->setEnabled(true);
	}
}
void byteXor::createConnections()
{
	connect(keyLine,SIGNAL(textEdited(const QString &)),this,SLOT(adjustKey()));
	connect(encryptButton,SIGNAL(clicked()),this,SLOT(encrypt()));
	connect(decryptButton,SIGNAL(clicked()),this,SLOT(decrypt()));
}
void byteXor::decrypt()
{
	QString title,tmp;
	title.append(tr("XOR decryption of <"));
	title.append(workTitle);
	title.append(tr(">, key <"));
	title.append(keyLine->text());
	title.append(">");
	workText.remove(' ');
	int len=workText.length(),i,k=0;
	int *key = new int[kLen];
	for(i=0;i<kLen;i++)
	{
		key[i]=0;
		if(keyLine->text()[i*3].isNumber())
			key[i]+=keyLine->text()[i*3].digitValue()*16;
		else
			key[i]+=(keyLine->text()[i*3].toAscii()-'A'+10)*16;
		if((i*3)+1>=keyLine->text().length())
			continue;
		if(keyLine->text()[(i*3)+1].isNumber())
			key[i]+=keyLine->text()[(i*3)+1].digitValue();
		else
			key[i]+=keyLine->text()[(i*3)+1].toAscii()-'A'+10;
	}
	for(i=0;i<len;i+=2)
	{
		int ch;
		if(workText[i].isNumber())
			ch=(workText[i].toAscii()-'0')*16;
		else
			ch=(workText[i].toAscii()-'A'+10)*16;
		if(workText[i+1].isNumber())
			ch+=(workText[i+1].toAscii()-'0');
		else
			ch+=(workText[i+1].toAscii()-'A'+10);
		tmp.append(QChar(ch^key[k]));
		k=(k+1)%kLen;
	}
	mw->newSubWindow(title,tmp);
	close();
}
void byteXor::encrypt()
{
	QString title,tmp;
	title.append(tr("XOR encryption of <"));
	title.append(workTitle);
	title.append(tr(">, key <"));
	title.append(keyLine->text());
	title.append(">");
	int len=workText.length(),i,k=0;
	int *key = new int[kLen];
	for(i=0;i<kLen;i++)
	{
		key[i]=0;
		if(keyLine->text()[i*3].isNumber())
			key[i]+=keyLine->text()[i*3].digitValue()*16;
		else
			key[i]+=(keyLine->text()[i*3].toAscii()-'A'+10)*16;
		if((i*3)+1>=keyLine->text().length())
			continue;
		if(keyLine->text()[(i*3)+1].isNumber())
			key[i]+=keyLine->text()[(i*3)+1].digitValue();
		else
			key[i]+=keyLine->text()[(i*3)+1].toAscii()-'A'+10;
	}
	for(i=0;i<len;i++)
	{
		int char1,char2;
		char1=(((workText[i].toAscii()^key[k])%256)/16);
		char1+=(char1>=10?'A'-10:'0');
		char2=(((workText[i].toAscii()^key[k])%256)%16);
		char2+=(char2>=10?'A'-10:'0');
		tmp.append(QChar(char1));
		tmp.append(QChar(char2));
		tmp.append(QChar(' '));
		k=(k+1)%kLen;
	}
	mw->newSubWindow(title,tmp);
	mw->currentMdiChild()->setCurrentFont(QFont("Fixed"));
	close();
}

#include "byteXor.moc"
