/***************************************************************************
*   Copyright (C) 2007 by Andrea Moro   apterium
*   andrea_moro@email.it                apterium@gmail.com
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 3 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
***************************************************************************/
#include <QtGui>
#include <ctype.h>
#include <gmpxx.h>
#include "hill.h"
#include "mdichild.h"
#include "mainWindow.h"
#include "Text.h"
#include "libClassicalCrypto/libClassicalCrypto.h"

hill::hill(mainWindow *main,QString text, QString title)
{
	setupUi(this);
	workText=text;
	workTitle=title;
	mw=main;
	descBox = new QGroupBox(tr("Description"),this);
	descLabel = new QLabel(descBox);
	char tmp[10];
	sprintf(tmp,"%d",mw->options.text_opt.alphabet.length());
	alphBox= new QGroupBox(tr("Used alphabet (")+ QString(tmp) +tr(" characters)"),this);
	matrixBox = new QGroupBox(tr("Hill key matrix"),this);
	alphM = new QRadioButton(tr("Alphabet characters"),matrixBox);
	numbM = new QRadioButton(tr("Number values"),matrixBox);
	alph = new QLineEdit(alphBox);
	alphMatrix = new QGroupBox(tr("Alphabet characters"),matrixBox);
	numbMatrix = new QGroupBox(tr("Number values"),matrixBox);
	QRegExp reA;
	if(mw->options.text_opt.keep_case)
		reA=QRegExp("[a"+mw->options.text_opt.alphabet+mw->options.text_opt.alphabet.toLower()+"]");
	else
		reA=QRegExp("[a"+mw->options.text_opt.alphabet+"]");
	QRegExpValidator *vA = new QRegExpValidator(reA,this);
	QRegExp reN("[0-9]*");
	QRegExpValidator *vN = new QRegExpValidator(reN,this);
	int i,j;
	for(i=0;i<10;i++)
		for(j=0;j<10;j++)
		{
			cellA[i][j] = new QLineEdit(alphMatrix);
			cellN[i][j] = new QLineEdit(numbMatrix);
			cellA[i][j]->setMaxLength(1);
			cellN[i][j]->setMaxLength(10);
			cellA[i][j]->setValidator(vA);
			cellN[i][j]->setValidator(vN);
			cellA[i][j]->setVisible(false);
			cellN[i][j]->setVisible(false);
		}
	rndMatrix = new QPushButton(tr("Generate &random key"),matrixBox);
	moltBox = new QGroupBox(tr("Multiplication variant"),this);
	variant1 = new QRadioButton(tr("(row vector) * (matrix)"),moltBox);
	variant2 = new QRadioButton(tr("(matrix) * (column vector)"),moltBox);
	sizeBox = new QGroupBox(tr("Size of matrix"),this);
	for(i=0;i<10;i++)
	{
		sprintf(tmp,"&%d x %d",i+1,i+1);
		sizeRadio[i]= new QRadioButton(QString(tmp),sizeBox);
	}
	sizeRadio[0]->setChecked(true);
	largerMatrix = new QPushButton(tr("&Larger matrix"),sizeBox);
	largerMatrix->resize(120,30);
	explainCheck = new QCheckBox(tr("Show details and single steps of the Hill cipher"), this);
	encButton = new QPushButton(tr("&Encrypt"),this);
	decButton = new QPushButton(tr("&Decrypt"),this);
	textOpt = new QPushButton(tr("&Text options"),this);
	cancButton = new QPushButton(tr("&Cancel"),this);
	littleHill();
	createConnections();
}
void hill::littleHill()
{
	largerMatrix->setText(tr("&Larger matrix"));
	big=false;
	this->resize(620,570);
	descBox->setGeometry(10,10,600,95);
	descLabel->move(10,20);
	descLabel->setText(tr(
			"The Hill cipher is a polygraphic substitution cipher based on linear algebra.\n"
			"This was the first polygraphic cipher, in which it was practical to operate on"
			"a group of letters\ncontaining more than three letters.\nThe length of the "
			"group of cleartext letters is the size of the quadratic matrix."));
	descBox->setVisible(true);
	alphBox->setGeometry(10,115,600,50);
	alph->setEnabled(false);
	alph->setText(mw->options.text_opt.alphabet);
	alph->setGeometry(10,20,580,20);
	alphBox->setVisible(true);
	matrixBox->setGeometry(10,175,400,300);
	matrixBox->setVisible(true);
	alphM->move(10,30);
	numbM->move(200,30);
	if((!alphM->isChecked())&&(!numbM->isChecked()))
		alphM->setChecked(true);
	alphM->setVisible(true);
	numbM->setVisible(true);
	alphMatrix->setGeometry(10,80,188,150);
	numbMatrix->setGeometry(202,80,188,150);
	int i,j;
	for(i=0;i<5;i++)
	{
		for(j=0;j<5;j++)
		{
			cellA[i][j]->setGeometry(10+(35*j),20+(25*i),30,20);
			cellN[i][j]->setGeometry(10+(35*j),20+(25*i),30,20);
			cellA[i][j]->setVisible(true);
			cellN[i][j]->setVisible(true);
		}
		for(;j<10;j++)
		{
			cellA[i][j]->setVisible(false);
			cellN[i][j]->setVisible(false);
		}
	}
	for(;i<10;i++)
		for(j=0;j<10;j++)
		{
			cellA[i][j]->setVisible(false);
			cellN[i][j]->setVisible(false);
		}
	rndMatrix->move(125,250);
	rndMatrix->setVisible(true);
	moltBox->setGeometry(415,175,195,70);
	variant1->move(10,20);
	if((!variant1->isChecked())&&(!variant2->isChecked()))
		variant1->setChecked(true);
	variant2->move(10,40);
	moltBox->setVisible(true);
	sizeBox->setGeometry(415,250,195,225);
	sizeBox->setVisible(true);
	for(i=0;i<5;i++)
		sizeRadio[i]->move(70,20+(30*i));
	for(;i<10;i++)
		sizeRadio[i]->setVisible(false);
	if(kLen>5)
		sizeRadio[4]->setChecked(true);
	largerMatrix->move(40,180);
	explainCheck->move(150,490);
	encButton->move(10,530);
	decButton->move(110,530);
	textOpt->move(350,530);
	cancButton->move(535,530);
	changeDim();
}
void hill::bigHill()
{
	largerMatrix->setText(tr("&Smaller matrix"));
	big=true;
	this->resize(975,670);
	descBox->setGeometry(10,10,955,95);
	descLabel->move(10,20);
	descLabel->setText(tr(
			"The Hill cipher is a polygraphic substitution cipher based on linear algebra.\n"
			"This was the first polygraphic cipher, in which it was practical to operate on"
			"a group of letters\ncontaining more than three letters.\nThe length of the"
			"group of cleartext letters is the size of the quadratic matrix."));
	descBox->setVisible(true);
	alphBox->setGeometry(10,115,755,50);
	alph->setEnabled(false);
	alph->setText(mw->options.text_opt.alphabet);
	alph->setGeometry(10,20,735,20);
	alphBox->setVisible(true);
	matrixBox->setGeometry(10,175,755,400);
	matrixBox->setVisible(true);
	alphM->move(10,30);
	numbM->move(200,30);
	alphM->setVisible(true);
	numbM->setVisible(true);
	alphMatrix->setGeometry(10,70,365,275);
	numbMatrix->setGeometry(380,70,365,275);
	int i,j;
	for(i=0;i<10;i++)
	{
		for(j=0;j<10;j++)
		{
			cellA[i][j]->setGeometry(10+(35*j),20+(25*i),30,20);
			cellN[i][j]->setGeometry(10+(35*j),20+(25*i),30,20);
			cellA[i][j]->setVisible(true);
			cellN[i][j]->setVisible(true);
		}
	}
	rndMatrix->move(300,360);
	rndMatrix->setVisible(true);
	moltBox->setGeometry(770,115,195,70);
	variant1->move(10,20);
	variant2->move(10,40);
	moltBox->setVisible(true);
	sizeBox->setGeometry(770,190,195,385);
	sizeBox->setVisible(true);
	for(i=0;i<10;i++)
	{
		sizeRadio[i]->move(70,30+(30*i));
		sizeRadio[i]->setVisible(true);
	}
	largerMatrix->move(40,340);
	explainCheck->move(180,590);
	encButton->move(10,630);
	decButton->move(210,630);
	textOpt->move(650,630);
	cancButton->move(835,630);
	changeDim();
}
void hill::changeDim()
{
	int i,j;
	for(i=0;i<10;i++)
		if(sizeRadio[i]->isChecked())
			kLen=i+1;
	for(i=0;i<kLen;i++)
	{
		for(j=0;j<kLen;j++)
		{
			if(alphM->isChecked())
			{
				cellA[i][j]->setEnabled(true);
				cellN[i][j]->setEnabled(false);
			}
			else
			{
				cellA[i][j]->setEnabled(false);
				cellN[i][j]->setEnabled(true);
			}
		}
		for(j=kLen;j<10;j++)
		{
			cellA[i][j]->setEnabled(false);
			cellN[i][j]->setEnabled(false);
		}
	}
	for(;i<10;i++)
		for(j=0;j<10;j++)
		{
			cellA[i][j]->setEnabled(false);
			cellN[i][j]->setEnabled(false);
		}
}
void hill::changeWin()
{
	if(big)
		littleHill();
	else
		bigHill();
}
void hill::updAlph()
{
	char tmp[10];
	sprintf(tmp,"%d",mw->options.text_opt.alphabet.length());
	alphBox->setTitle(tr("Used alphabet (")+ QString(tmp) +tr(" characters)"));
	alph->setText(mw->options.text_opt.alphabet);
	QString alphabet=mw->options.text_opt.alphabet;
	int startN=mw->options.text_opt.aEqual1?1:0;
	int i,j,pos;
	for(i=0;i<10;i++)
		for(j=0;j<10;j++)
			if((cellA[i][j]->text().length()!=0)&&
				((pos=alphabet.indexOf(cellA[i][j]->text())+startN)!=-1))
			{
				sprintf(tmp,"%d",alphabet.indexOf(cellA[i][j]->text())+startN);
				cellN[i][j]->setText(QString(tmp));
			}
}
void hill::createConnections()
{
	int i,j;
	for(i=0;i<10;i++)
		for(j=0;j<10;j++)
		{
			connect(cellA[i][j],SIGNAL(textEdited(const QString&)),this,SLOT(upMatr()));
			connect(cellN[i][j],SIGNAL(textEdited(const QString&)),this,SLOT(upMatr()));
		}
	connect(encButton,SIGNAL(clicked()),this,SLOT(encrypt()));
	connect(decButton,SIGNAL(clicked()),this,SLOT(decrypt()));
	for(i=0;i<10;i++)
		connect(sizeRadio[i],SIGNAL(clicked()),this,SLOT(changeDim()));
	connect(rndMatrix,SIGNAL(clicked()),this,SLOT(randKey()));
	connect(textOpt,SIGNAL(clicked()),mw,SLOT(opt_text()));
	connect(largerMatrix,SIGNAL(clicked()),this,SLOT(changeWin()));
	connect(cancButton,SIGNAL(clicked()),this,SLOT(close()));
	connect(alphM,SIGNAL(clicked()),this,SLOT(changeDim()));
	connect(numbM,SIGNAL(clicked()),this,SLOT(changeDim()));
	connect(mw->opttext_changed,SIGNAL(triggered()),this,SLOT(updAlph()));
}
bool hill::checkKey()
{
	int tmp=0,i,j;
	int startN=mw->options.text_opt.aEqual1?1:0;
	if(alphM->isChecked())
	{
		for(i=0;i<kLen;i++)
			for(j=0;j<kLen;j++)
				tmp+=cellA[i][j]->text().length();
	}
	else
	{
		for(i=0;i<kLen;i++)
			for(j=0;j<kLen;j++)
			{
				if((cellN[i][j]->text().toInt()-startN>=0)&&
				(cellN[i][j]->text().toInt()-startN<mw->options.text_opt.alphabet.length()))
					tmp++;
			}
	}
	if(tmp!=kLen*kLen)
		return false;
	return true;
}
void hill::upMatr()
{

	QLineEdit *tmp=qobject_cast<QLineEdit*>(sender());
	if(tmp->text().length()!=0)
	{
		QString alphabet=mw->options.text_opt.alphabet;
		int startN=mw->options.text_opt.aEqual1?1:0;
		int nextFocus = 1;
		if(alphM->isChecked())
		{
			if(mw->options.text_opt.keep_case)
				tmp->setText(tmp->text().toUpper());
			if(alphabet.indexOf(tmp->text())==-1)
			{
				nextFocus=0;
				tmp->setText("");
			}
			int i,j,f=0;
			for(i=0;i<10;i++)
			{
				for(j=0;j<10;j++)
					if(cellA[i][j]==tmp)
					{
						f=1;
						break;
					}
				if(f)
					break;
			}
			int idx=alphabet.indexOf(tmp->text());
			if(nextFocus)
			{
				char str[10];
				sprintf(str,"%d",idx+startN);
				cellN[i][j]->setText(str);
			}
			else
				cellN[i][j]->setText("");
		}
		else
		{
			int i,j,f=0;
			for(i=0;i<10;i++)
			{
				for(j=0;j<10;j++)
					if(cellN[i][j]==tmp)
					{
						f=1;
						break;
					}
				if(f)
					break;
			}
			if((cellN[i][j]->text().toInt()-startN>=0)&&
				(cellN[i][j]->text().toInt()-startN<alphabet.length()))
				cellA[i][j]->setText(QChar(alphabet[cellN[i][j]->text().toInt()-startN]));
			else
			{
				nextFocus=0;
				cellA[i][j]->setText("");
			}
		}
		if(nextFocus)
			focusNextChild();
	}
	else
	{
		if(alphM->isChecked())
		{
			int i,j,f=0;
			for(i=0;i<10;i++)
			{
				for(j=0;j<10;j++)
					if(cellA[i][j]==tmp)
					{
						f=1;
						break;
					}
				if(f)
					break;
			}
			cellN[i][j]->setText("");
		}
		else
		{
			int i,j,f=0;
			for(i=0;i<10;i++)
			{
				for(j=0;j<10;j++)
					if(cellN[i][j]==tmp)
					{
						f=1;
						break;
					}
				if(f)
					break;
			}
			cellA[i][j]->setText("");
		}
	}

}
int* hill::ext_gcd(int a, int b)
{
	int t,x=0,y=1,lastx=1,lasty=0,q;
	int *vet = new int[3];
	while(b)
	{
		t=b;
		q=a/b;
		b=a%b;
		a=t;
		t=x;
		x=lastx-q*x;
		lastx=t;
		t=y;
		y=lasty-q*y;
		lasty=t;
	}
	vet[0]=a;
	vet[1]=lastx;
	vet[2]=lasty;
	return vet;
}
bool hill::inverse(int **matrix,int modify)
{
	if(!matrix)
		return false;
	int inverse[kLen][kLen],alphLen=mw->options.text_opt.alphabet.length();
	int matrixTmp[kLen][kLen];
	int row,col,pivot,i,j,tmp;
	int inverses[alphLen];
	int *tmpV;
	inverses[0]=0;
	inverses[1]=1;
	for(i=2;i<alphLen;i++)
	{
		if((tmpV=ext_gcd(alphLen,i))[0]==1)
		{
			inverses[i]=tmpV[2]>0?tmpV[2]:(alphLen+tmpV[2]);
		}
		else
			inverses[i]=0;
	}
	for(i=0;i<kLen;i++)
		for(j=0;j<kLen;j++)
			matrixTmp[i][j]=matrix[i][j];
	for(row=0;row<kLen;row++)
	{
		for(col=0;col<kLen;col++)
			inverse[row][col]=0;
		inverse[row][row]=1;
	}
	for(row=0;row<kLen;row++)
	{
		if (inverses[matrixTmp[row][row]] == 0)
		{
			for (i = row + 1; i < kLen; i++)
				if (inverses[matrixTmp[i][row]])
				{
// swap_row(i,row) in matrixTmp and in inverse
					for (j = 0; j < kLen; j++)
					{
						tmp = matrixTmp[i][j];
						matrixTmp[i][j] = matrixTmp[row][j];
						matrixTmp[row][j] = tmp;
						tmp = inverse[i][j];
						inverse[i][j] = inverse[row][j];
						inverse[row][j] = tmp;
					}
					break;
				}
			if (i == kLen)
				return false;
		}
		pivot = inverses[matrixTmp[row][row]];
// multiply all terms on row 'row' of matrixTmp and inverse
// with the multiplicative inverse of matrixTmp[row][row]
		for (j = 0; j < kLen; j++)
		{
			matrixTmp[row][j] = (matrixTmp[row][j] * pivot) % alphLen;
			inverse[row][j] = (inverse[row][j] * pivot) % alphLen;
		}
// change to 0 all the elements of the column 'row' except the element of the row 'row'
// in the matrix 'matrixTmp' and consequently change the matrix 'inverse'
		for (i = 0; i < kLen; i++)
			if (i != row)
			{
				pivot = matrixTmp[i][row];
				for (j = 0; j < kLen; j++)
				{
					matrixTmp[i][j] -= (pivot * matrixTmp[row][j]) % alphLen;
					if (matrixTmp[i][j] < 0)
						matrixTmp[i][j] += alphLen;
					inverse[i][j] -= (pivot * inverse[row][j]) % alphLen;
					if (inverse[i][j] < 0)
						inverse[i][j] += alphLen;
				}
			}
	}
	if(modify)
		for(row=0;row<kLen;row++)
			for(col=0;col<kLen;col++)
				matrix[row][col]=inverse[row][col];
	return true;
}
void hill::randKey()
{
	int i,j,rnd;
	int **numMatr=new int*[kLen];
	int startN = mw->options.text_opt.aEqual1?1:0;
	QString alph = mw->options.text_opt.alphabet;
	for(i=0;i<kLen;i++)
		numMatr[i]=new int[kLen];
	while(true)
	{
		for(i=0;i<kLen;i++)
			for(j=0;j<kLen;j++)
			{
				rnd=(int)(((double)qrand()/RAND_MAX)*alph.length());
				cellA[i][j]->setText(QChar(alph[rnd]));
				char tmp[10];
				sprintf(tmp,"%d",rnd+startN);
				cellN[i][j]->setText(tmp);
				numMatr[i][j]=rnd;
			}
		if(inverse(numMatr,0))
		{
			for(i=0;i<kLen;i++)
				delete numMatr[i];
			delete numMatr;
			return;
		}
	}
}
void hill::encrypt()
{
	this->ciph(ciphFlag(ENCIPHER));
}

void hill::decrypt()
{
	this->ciph(ciphFlag(DECIPHER));
}

void hill::ciph(const ciphFlag& flag)
{
		if(!checkKey())
	{
		QMessageBox::warning(this, tr("Error"),
			tr("All the fields of the key must contain a character."));
		return;
	}
	
	QString title, key;
	int **numMatr = new int*[kLen];
	int startN = mw->options.text_opt.aEqual1?1:0;
	
	for(int i=0;i<kLen;i++)
	{
		numMatr[i]=new int[kLen];
		for(int j=0;j<kLen;j++)
		{
			key.append(cellA[i][j]->text());
			numMatr[i][j]=cellN[i][j]->text().toInt()-startN;
		}
		key.append(' ');
	}

	switch (flag) {
	case ciphFlag(DECIPHER):
		title.append(tr("Hill decryption of <"));
	break;
	case ciphFlag(ENCIPHER):
		title.append(tr("Hill encryption of <"));
	break;
	default:
		#ifdef DEBUG
		std::cerr << std::endl << "Line " << __LINE__ << " of " << __FILE__  << std::endl;
		std::cerr << "Third parameter of a two-value enumeration ???" << std::endl;
                #endif
		abort();
		break;
	}
	title.append(workTitle);
	title.append(tr(">, key <"));
	title.append(key);
	title.append(">");
	

	QString alphabet = mw->options.text_opt.alphabet;
	
	LibClassicalCrypto::HillEngine* engine;
	try {
		engine = new LibClassicalCrypto::HillEngine(numMatr, mpz_class(alphabet.size()), kLen);
	}
	catch (LibClassicalCrypto::InvalidKeyException& e) {
		delete[] numMatr;
		QMessageBox::warning(this, tr("Error"),
			tr("The given matrix is not invertible !"));
		return;
	}
	QString text = QString(workText);

	Qt::CaseSensitivity cs;
	if(mw->options.text_opt.keep_case)
		cs = Qt::CaseInsensitive;
	else
		cs = Qt::CaseSensitive;

	Text theText(text, &alphabet, cs);
	
	//padding of the text, with 0 at the end : 
	std::vector<mpz_class>::size_type l = theText.getWorkTextLength();
	while ( l % kLen != 0) {
		theText.addZero();
		l++;
	}
	switch(flag) {
	case ciphFlag(DECIPHER):
		theText.setResultingText(engine->decipher(theText.getIntRepr()));
	break;
	case ciphFlag(ENCIPHER):
		theText.setResultingText(engine->encipher(theText.getIntRepr()));
	break;
	default:
		#ifdef DEBUG
		std::cerr << std::endl << "Line " << __LINE__ << " of " << __FILE__  << std::endl;
		std::cerr << "Third parameter of a two-value enumeration ???" << std::endl;
                #endif
		break;
	}
	QString newWorkText = theText.getResultingText();
	delete engine;
	
	mw->newSubWindow(title, newWorkText);
	if(explainCheck->isChecked())
		explain();
	close();
}



void hill::explain()
{
	QString title,key,explanation,plainText,tmp,cryptoText;
	QString alph = mw->options.text_opt.alphabet;
	int k,sum,i,j;
	int** numMatr = new int*[kLen];
	int startN=mw->options.text_opt.aEqual1?1:0;
	char crypt[11]="CIPHERTEXT",str[10];

	for(i=0;i<kLen;i++)
	{
		numMatr[i]=new int[kLen];
		for(j=0;j<kLen;j++)
		{
			key.append(cellA[i][j]->text());
			numMatr[i][j]=cellN[i][j]->text().toInt()-startN;
		}
		key.append(' ');
	}
	title.append(tr("Hill cipher explanation, key <"));
	title.append(key);
	title.append(">[*]");
	explanation.append(tr("HILL encryption / decryption\n\nThe "
				"alphabet is encoded into such as:\n"));
	for(i=0;i<alph.length();i++)
	{
		sprintf(str,"%03d",i);
		explanation.append(alph[i]+" --> "+str+"\t");
		if((i+1)%4==0)
			explanation.append("\n");
	}
	explanation.append("\n\n\n");
	explanation.append(tr("Hill encryption:\n\n"
			"The selected Hill matrix for encryption is:\n\nH[t] =\n"));
	for(i=0;i<kLen;i++)
	{
		explanation.append("\t[ ");
		for(j=0;j<kLen;j++)
		{
			explanation.append(cellA[i][j]->text());
			explanation.append(" ");
		}
		explanation.append("]\n");
	}
	explanation.append(tr("\nThis matrix encoded into numbers:\n\nH[n] =\n"));

	for(i=0;i<kLen;i++)
	{
		explanation.append("\t[ ");
		for(j=0;j<kLen;j++)
		{
			sprintf(str,"%03d",numMatr[i][j]);
			explanation.append(str);
			explanation.append(" ");
		}
		explanation.append("]\n");
	}
	explanation.append(tr("\nExample: Encryption of the plaintext: \""));
	for(i=0;i<kLen;i++)
		explanation.append(crypt[i]);
	explanation.append(tr("\"\n\nPlaintext vector K encoded into numbers:\n\n"));
	explanation.append("K[n] = [ ");
	for(i=0;i<kLen;i++)
	{
		sprintf(str,"%03d",alph.indexOf(crypt[i]));
		explanation.append(str);
		explanation.append(" ");
	}
	explanation.append(tr("]\n\nComputation of the Hill cipher by the matrix vector "));
	explanation.append(tr("multiplication  K[n]*H[n]:\n\n"));
	for(k=0;k<kLen;k++)
	{
		sum=0;
		for(j=0;j<kLen;j++)
		{
			sprintf(str,"%03d",alph.indexOf(crypt[j]));
			explanation.append(str);
			explanation.append('*');
			sprintf(str,"%03d",numMatr[j][k]);
			explanation.append(str);
			sum+=(alph.indexOf(crypt[j]))*(numMatr[j][k]);
			if(j!=kLen-1)
				explanation.append(" + ");
		}
		sum=(sum%alph.length());
		sprintf(str,"%03d",alph.length());
		explanation.append("  (mod "+QString(str)+") = ");
		sprintf(str,"%03d",sum);
		explanation.append(str);
		explanation.append(" --> ");
		cryptoText.append(alph[sum]);
		explanation.append(alph[sum]);
		explanation.append("\n");
	}

	explanation.append(tr("\nThe Hill ciphertext is: \""));
	explanation.append(cryptoText);
	explanation.append(tr("\"\n\n\nHill decryption:\n\n"));
	explanation.append(tr("The selected inverse Hill matrix "
				"for decryption is:\n\nD[t] =\n"));
	

	try {
		QString alphabet = mw->options.text_opt.alphabet;
		LibClassicalCrypto::HillEngine *engine = 
			new LibClassicalCrypto::HillEngine(numMatr, mpz_class(alphabet.size()), kLen);
		engine->getInverseKey(numMatr, kLen);
		delete engine;
	}
	catch (LibClassicalCrypto::InvalidKeyException& e) {
		delete[] numMatr;
		QMessageBox::warning(this, tr("Error"),
		     tr("The given matrix is not invertible !"));
		return;
	}
	
	
	for(i=0;i<kLen;i++)
	{
		explanation.append("\t[ ");
		for(j=0;j<kLen;j++)
		{
			explanation.append(QString(alph[numMatr[i][j]]));
			explanation.append(" ");
		}
		explanation.append("]\n");
	}
	explanation.append(tr("\nThis matrix encoded into numbers:\n\nD[n] =\n"));
	for(i=0;i<kLen;i++)
	{
		explanation.append("\t[ ");
		for(j=0;j<kLen;j++)
		{
			sprintf(str,"%03d",numMatr[i][j]);
			explanation.append(str);
			explanation.append(" ");
		}
		explanation.append("]\n");
	}
	explanation.append(tr("\nExample: Decryption of the ciphertext: \""));
	explanation.append(cryptoText);
	explanation.append(tr("\":\n\nCiphertext vector C encoded into numbers:\n\nC[n] = [ "));
	for(i=0;i<kLen;i++)
	{
		sprintf(str,"%03d",alph.indexOf(cryptoText[i]));
		explanation.append(str);
		explanation.append(" ");
	}
	explanation.append(tr("]\n\nComputation of the Hill decryption by the matrix vector "));
	explanation.append(tr("multiplication  C[n]*D[n]:\n\n"));
	for(k=0;k<kLen;k++)
	{
		sum=0;
		for(j=0;j<kLen;j++)
		{
			sprintf(str,"%03d",alph.indexOf(cryptoText[j]));
			explanation.append(str);
			explanation.append('*');
			sprintf(str,"%03d",numMatr[j][k]);
			explanation.append(str);
			sum+=(alph.indexOf(cryptoText[j]))*(numMatr[j][k]);
			if(j!=kLen-1)
				explanation.append(" + ");
		}
		sum=(sum%alph.length());
		sprintf(str,"%03d",alph.length());
		explanation.append("  (mod "+QString(str)+") = ");
		sprintf(str,"%03d",sum);
		explanation.append(str);
		explanation.append(" --> ");
		plainText.append(alph[sum]);
		explanation.append(alph[sum]);
		explanation.append("\n");
	}
	explanation.append(tr("\nThe Hill plaintext is: \""));
	explanation.append(plainText);
	explanation.append("\"\n\n\n");
	mw->newSubWindow(title,explanation);
	mw->currentMdiChild()->setFont(QFont("Monospace"));
}

#include "hill.moc"
