/***************************************************************************
 *   Copyright (C) 2007 by apterium   *
 *   apterium@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "CustomFactorisation.h"
#include "../libNumberTheory/libNumberTheory.h"

namespace libAnalysis
{
	CustomFactorisation::CustomFactorisation(const mpz_class& n)
	{
		this->indexes = new std::list<FactorisationAlgorithmIndex>;
		//to be updated when a new algorithm is added
		this->indexes->push_front(FERMAT);
		
		//TODO : initialise the proper factoring engine, given the size
		//of the number to factorise. Add a new method...
		//if  LOWER_BORDER< number->bitSize() < UPPER_BORDER
		//create the appropriated factorisation engine
		//get the next factor.
		this->engine = new FactorisationFermat(n);
	}
	
	CustomFactorisation::CustomFactorisation(const mpz_class& n, const std::list<FactorisationAlgorithmIndex>& l) 
	{
		this->indexes = new std::list<FactorisationAlgorithmIndex>(l);
		this->engine = new FactorisationFermat(n);
	}

	CustomFactorisation::~CustomFactorisation()
	{
		delete this->indexes;
		//TODO : MEMORY LEAK or NOT ? 
		delete this->engine;
	}
	
	
	
	mpz_class CustomFactorisation::getNextFactor() throw (IsPrimeException)
	{
		if (this->primalityTest() >=1) {
			throw (IsPrimeException());
		}
		//TODO : take care of the found factor and quotients, which are
		//not necessarily prime ! So better get it in priority queue.
		return this->engine->getNextFactor();
	}
	
	mpz_class CustomFactorisation::getRemainingQuotient()
	{
		return this->engine->getRemainingQuotient();
	}
	
	int CustomFactorisation::isPrime()
	{
		//Add the code specifying which primality tests we are allowed
		//to use here, instead of this hardcoded use of the GMP
		//primality test.
		return this->primalityTest();
	}
	
	int CustomFactorisation::primalityTest() {
		libNumberTheory::GMPCompositenessTest test = libNumberTheory::GMPCompositenessTest(this->getRemainingQuotient());
		return test.isComposite();
	}

	
} //namespace libAnalysis
