/***************************************************************************
 *   Copyright (C) 2007 by apterium   *
 *   apterium@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef __LIB_ANALYSIS_FACTORISATION_H
#define __LIB_ANALYSIS_FACTORISATION_H

#include <gmp.h>
#include <gmpxx.h>
#include "IsPrimeException.h"

namespace libAnalysis
{
	
	/**
	 * Generic factorisation class. 
	 *
	 * @author apterium
	 * @todo Redesign the class : first do the computation and compute a
	 * non-trivial factor, then allow it to be get.
	 */ 
	class Factorisation
	{
	public:
		//NOTE : EVERY SUBCLASS SHOULD REALLY PROPOSE ONLY ONE
		//CONSTRUCTOR, ACCEPTING THE INTEGER TO FACTOR AS THE ONLY
		//PARAMETER.

		virtual ~Factorisation();
		
		/**
		 * Gets the next factor of the given number. The given
		 * number is divided by it, so don't expect to be able to
		 * get it again. 
		 * Note that you should know this number is composite before
		 * using such a method, because (1) Using exception for a normal
		 * case is a bad thing, and (2) It's much slower than
		 * primality/compositeness tests.
		 * Note too that the factor found is not necessarily prime ! 
		 *
		 * @return the next factor computed by the algorithm.
		 * @throw IsPrimeException if the current result is a prime or
		 * on the conditions specified in the specific implementation.
		 */
		virtual mpz_class getNextFactor() throw (IsPrimeException) = 0;
		
		/**
		 * Gets the remaining quotient which is to be factorised.
		 * 
		 */
		virtual mpz_class getRemainingQuotient() = 0;
		
	private:
		
		
	};
}

#endif //__LIB_ANALYSIS_FACTORISATION_H
