/***************************************************************************
 *   Copyright (C) 2007 by apterium   *
 *   apterium@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <iostream>
#include "FactorisationBruteforce.h"

namespace libAnalysis
{
	FactorisationBruteforce::FactorisationBruteforce(const mpz_class& n)
	{
		this->current = new mpz_class(n);
		this->lastFactor = new mpz_class(2);
		this->squareRootBound = new mpz_class(sqrt(n) + 1) ;
	}
	
	
	FactorisationBruteforce::~FactorisationBruteforce()
	{
		delete this->current;
		delete this->lastFactor;
		delete this->squareRootBound;
	}
	
	mpz_class FactorisationBruteforce::getNextFactor() throw(IsPrimeException)
	{
		if (*(this->current) <= 3) {
			throw IsPrimeException();
		}
		//NOTE : NO NEED TO KEEP THE SQUARE ROOT BOUND !!!
		//JUST GO ON FROM THE LAST FACTOR !

		while ((*(this->lastFactor) < *(this->squareRootBound)) && 
		       (*(this->current) % *(this->lastFactor) != 0)) {
			*(this->lastFactor) += 1;
		}
		
		if (*(this->lastFactor) != *(this->squareRootBound)) {
			//NOTE : may be fastened using GMP C routines, as the
			//division is known to be exact. We maintain clarity of
			//the code at the expense of possible running time.
			*(this->current) /= *(this->lastFactor);
			return *(this->lastFactor);
		}
		else {
			//no factor was found
			throw IsPrimeException();
		}
	}
	
	mpz_class FactorisationBruteforce::getRemainingQuotient()
	{
		return *(this->current);
		
	}

} //namespace libAnalysis
