/***************************************************************************
 *   Copyright (C) 2007 by apterium   *
 *   apterium@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef __LIB_ANALYSIS_FACTORISATION_FERMAT_H
#define __LIB_ANALYSIS_FACTORISATION_FERMAT_H

#include "Factorisation.h"

namespace libAnalysis
{
	
	/**
	 * Fermat factorisation of an integer. 
	 *
	 * @author apterium
	 * 
	 */ 
	class FactorisationFermat : public Factorisation
	{
	public:
		/**
		 * Constructor. Constructs a new factoring engine, given the
		 * integer to be factorised.
		 * 
		 * @param n a constant reference to the integer to factor. 
		 */
		FactorisationFermat(const mpz_class& n);

		/**
		 * Destructor.
		 * 
		 */
		~FactorisationFermat();
		
		/**
		 * Retrieves the next smallest factor of the initial number.
		 *
		 * @throw IsPrimeException if the current result is a prime or
		 * is less than 3.
		 * @see Factorisation
		 */
		mpz_class getNextFactor() throw (IsPrimeException);

		/**
		 * @see Factorisation
		 * 
		 */
		mpz_class getRemainingQuotient();
		
	private:
		/**
		 * The remaining factor of the initial number, which is about to
		 * be factorised.
		 *
		 */
		mpz_class* current;
			
		/**
		 * The upper bound for a, according to "Primes Numbers, A Computational Perspective"
		 *
		 */
		mpz_class* upperBound;
	};
}

#endif //__LIB_ANALYSIS_FACTORISATION_BRUTEFORCE_H
