/***************************************************************************
 *   Copyright (C) 2007 by apterium   *
 *   apterium@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef __LIB_ANALYSIS_TEST_H
#define __LIB_ANALYSIS_TEST_H

#include "libAnalysis.h"

using namespace libAnalysis;

/**
 * Runs all tests on the whole libAnalysis library.
 *
 *
 * @author apterium
 * @todo Provide arguments in order to tell the implementation which
 * tests should be ran, when the library will become bigger and
 * bigger.
 * @return the error code of the failed test, or 0 if no.
 */
int main(int argc, char** argv);

/**
 * Tests the bruteforce factorisation method.
 *
 * @param n the number to be factorised.
 */
int testBruteforceFactoring(const mpz_class& n);

/**
 * Tests the Fermat factorisation method.
 *
 * @param n the number to be factorised.
 */
int testFermatFactoring(const mpz_class& n);

/**
 * Tests the custom factorisation method. Take care about this test when adding
 * new algorithms !!
 *
 *
 * @param n the number to be factorised.
 */
int testCustomFactoring(const mpz_class& n);

	


#endif //__LIB_ANALYSIS_TEST_H
