/***************************************************************************
 *   Copyright (C) 2007 by apterium   *
 *   apterium@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef __LIB_CLASSICAL_ANALYSIS_CAESAR_ANALYSIS_H
#define __LIB_CLASSICAL_ANALYSIS_CAESAR_ANALYSIS_H

#include <vector>
#include <gmpxx.h>

namespace libClassicalAnalysis
{
	/**
	 * @short Caesar cipher cryptanalysis. 
	 * This class provides the whole Caesar cipher cryptanalysis, given a
	 * cipher text. That is, a frequency analysis, the autocorrelation
	 * computation given a reference text/frequency set, and most probable
	 * key derivation.
	 * 
	 * @author apterium
	 */
	class CaesarAnalysis
	{
	public:
		/**
		 * Constructs a new CaesarAnalysis, given a reference text.
		 *
		 * @param cipherText the cipher text to analyse.
		 * @param alphabet the alphabet (letters to take into account)
		 * numerical representation. 
		 * @param referenceText the reference text for analysing the
		 * frequencies.
		 * @todo not very useful ? the reference frequencies should be
		 * common to the whole cryptoolinux...
		 */
		//		static CaesarAnalysis giveReferenceText(const std::vector<mpz_class>& cipherText,
		//							const std::vector<mpz_class>& alphabet,
		//							const std::vector<mpz_class>& referenceText);
		
		/**
		 * Constructs a new CaesarAnalysis, given a reference text.
		 *
		 * @param cipherText the cipher text to analyse.
		 * @param alphabet the alphabet numerical representation. It
		 * shall contain every letter which should be taken into account.
		 * @param referenceFrequencies the reference frequencies for
		 * their analysis. This vector SHALL be of the same length as
		 * the alphabet.
		 */
		//		static CaesarAnalysis giveReferenceFrequencies(const std::vector<mpz_class>& cipherText,
		//                                                      const std::vector<mpz_class>& alphabet,
		//							const std::vector<mpz_class>& referenceFreq);
		
		/**
		 * Constructs a new CaesarAnalysis instance.
		 * 
		 * @param text the text
		 * @param alphabet the alphabet
		 * @param referenceFreq the reference frequencies. That is, a vector
		 * of the same length as the alphabet, containing the
		 * frequencies of every letter.
		 */
		CaesarAnalysis(const std::vector<mpz_class>& text, 
			       const std::vector<mpz_class>& alphabet,
			       const std::vector<mpz_class>& referenceFreq);

		/**
		 * Destructor.
		 * 
		 */
		~CaesarAnalysis();

	private:
		/**
		 * The text.
		 *
		 */
		std::vector<mpz_class> *theText;
		
		/**
		 * The alphabet.
		 *
		 */
		std::vector<mpz_class> *theAlphabet;
		
		/**
		 * The reference frequencies.
		 *
		 */
		std::vector<mpz_class> *referenceFrequencies;
	};

}


#endif // __LIB_CLASSICAL_ANALYSIS_CAESAR_ANALYSIS_H
