/***************************************************************************
 *   Copyright (C) 2007 by apterium   *
 *   apterium@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include <gmp.h>
#include "Frequencies.h"

namespace libClassicalAnalysis
{
	Frequencies::Frequencies(const std::vector<mpz_class>& t,
				 const std::vector<mpz_class>& alph)
	{
		this->text = new std::vector<mpz_class>(t);
		this->alphabet = new std::vector<mpz_class>(alph);
	}
	
	Frequencies::~Frequencies()
	{
		delete this->text;
		delete this->alphabet;
	}
	
	std::vector<mpz_class> Frequencies::getSingleFrequencies()
	{
		std::vector<mpz_class>::size_type i,j = 0;
		std::vector<mpz_class> ret;
		ret.reserve(this->alphabet->size());
		//naive algorithm
		for (i = 0 ; i < this->alphabet->size() ; i++) {
			mpz_class counter = 0;
			for (j = 0 ; j < this->text->size() ; j++) {
				if (this->text->at(j) == this->alphabet->at(i)) {
					counter += 1;
				}
			}
			ret.push_back(counter);
		}
		return ret;
	}
	
	std::vector<std::vector<mpz_class> > Frequencies::getTwoFrequencies()
	{
		std::vector<std::vector<mpz_class> >::size_type i, j, k;
		std::vector<std::vector<mpz_class> > ret;
		ret.reserve(this->alphabet->size());
		
		//simple, not optimised, run through the whole text.
		for (i = 0 ; i < this->alphabet->size() ; i++) {
			ret.push_back(std::vector<mpz_class>());
			ret.at(i).reserve(this->alphabet->size());
			
			for (j = 0 ; j < this->alphabet->size() ; j++) {
				mpz_class counter = 0;
				for (k = 0 ; k < this->text->size() - 1 ; k++) {
					if (this->text->at(k) == this->alphabet->at(i) && 
					    this->text->at(k + 1) == this->alphabet->at(j) ){
						counter++;
					}
				}
				ret.at(i).push_back(counter);
			}
		}
		return ret;
	}
	
	//TODO : add a function to compute every repetition, whatever their
	//size. Not in a table because the table will mainly contain zeroes, but
	//sometimes a long repetition may be found (for example, 5 of 6 characters).
}
