/*************************************************************************** 
 *   Copyright (C) 2007 by apterium   *                                      
 *   apterium@gmail.com   *                                                  
 *                                                                         * 
 *   This program is free software; you can redistribute it and/or modify  * 
 *   it under the terms of the GNU General Public License as published by  * 
 *   the Free Software Foundation; either version 3 of the License, or     * 
 *   (at your option) any later version.                                   * 
 *                                                                         * 
 *   This program is distributed in the hope that it will be useful,       * 
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        * 
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         * 
 *   GNU General Public License for more details.                          * 
 *                                                                         * 
 *   You should have received a copy of the GNU General Public License     * 
 *   along with this program; if not, write to the                         * 
 *   Free Software Foundation, Inc.,                                       * 
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             * 
 ***************************************************************************/

#include "CaesarEngine.h"

namespace LibClassicalCrypto
{
	CaesarEngine::CaesarEngine(const mpz_class& k, const mpz_class& modulus)
	{
		this->ring = new libalgebra::RingZn(modulus);
		this->key = new libalgebra::RingZnElement(k, this->ring);
	}
	
	CaesarEngine::~CaesarEngine()
	{
		delete this->ring;
		delete this->key;
	}
	
	std::vector<mpz_class> CaesarEngine::encipher(const std::vector<mpz_class>& plainText) const
	{
		std::vector<libalgebra::RingZnElement> cipherText =  this->computeCipher(this->toRingZnElement(plainText, this->ring) );
		return this->toMpz_class(cipherText);
	}
	
	std::vector<mpz_class> CaesarEngine::decipher(const std::vector<mpz_class>& cipherText) const
	{
		std::vector<libalgebra::RingZnElement> plainText = this->computeDecipher(this->toRingZnElement(cipherText, this->ring));
		return this->toMpz_class(plainText);
	}
	
	std::vector<libalgebra::RingZnElement> CaesarEngine::computeCipher(const std::vector<libalgebra::RingZnElement>& plainText) const
	{
		std::vector<libalgebra::RingZnElement> ret;
		ret.reserve(plainText.size());
		for (std::vector<libalgebra::RingZnElement>::size_type i = 0 ; i < plainText.size() ; i++) {
			ret.push_back(libalgebra::RingZnElement(plainText.at(i) + *(this->key)) ) ;
		}		

		return ret;
	}

	std::vector<libalgebra::RingZnElement> CaesarEngine::computeDecipher(const std::vector<libalgebra::RingZnElement>& cipherText) const
	{
		std::vector<libalgebra::RingZnElement> ret;
		ret.reserve(cipherText.size());
		for (std::vector<libalgebra::RingZnElement>::size_type i = 0 ; i < cipherText.size() ; i++) {
			ret.push_back(libalgebra::RingZnElement(cipherText.at(i) - *(this->key)) ) ;
		}		
		return ret;
	}
	
	
} //LibClassicalCrypto
