/*************************************************************************** 
 *   Copyright (C) 2007 by apterium   *                                      
 *   apterium@gmail.com   *                                                  
 *                                                                         * 
 *   This program is free software; you can redistribute it and/or modify  * 
 *   it under the terms of the GNU General Public License as published by  * 
 *   the Free Software Foundation; either version 3 of the License, or     * 
 *   (at your option) any later version.                                   * 
 *                                                                         * 
 *   This program is distributed in the hope that it will be useful,       * 
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        * 
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         * 
 *   GNU General Public License for more details.                          * 
 *                                                                         * 
 *   You should have received a copy of the GNU General Public License     * 
 *   along with this program; if not, write to the                         * 
 *   Free Software Foundation, Inc.,                                       * 
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             * 
 ***************************************************************************/

#ifndef __LIB_CLASSICAL_CRYPTO_CAESAR_ENGINE_H
#define __LIB_CLASSICAL_CRYPTO_CAESAR_ENGINE_H

#include "../libalgebra/libalgebra.h"
#include "CipherEngine.h"

namespace LibClassicalCrypto
{
	/**
	 * @short Caesar/Rot13 cipher engine.
	 *
	 * @author apterium
	 */
	class CaesarEngine : public CipherEngine
	{
	public:
		/**
		 * Constructs a new CaesarEngine, given the aphabet length and
		 * the key (the shift).
		 * 
		 * @param k a constant reference to the key, that is the shift
		 * of every enciphered character.
		 * @param moduluas a constant reference to the length of the
		 * alphabet.
		 */
		CaesarEngine(const mpz_class& k, const mpz_class& modulus);
		
		/**
		 * Destructor.
		 * 
		 */
		virtual ~CaesarEngine();
		
		/**
		 * @see CipherEngine
		 * 
		 */
		std::vector<mpz_class> encipher(const std::vector<mpz_class>& plainText) const;
		
		/**
		 * @see CipherEngine
		 * 
		 */
		std::vector<mpz_class> decipher(const std::vector<mpz_class>& cipherText)  const;
	private:
		/**
		 * The Ring which the oerations will take place in.
		 * 
		 */
		libalgebra::RingZn* ring;
		
		/**
		 * The key.
		 *
		 */
		libalgebra::RingZnElement* key;
		
		/**
		 * Computes the cipher text associated with the plaintext, using
		 * the RingZnElements.
		 * 
		 * @param plainText the plainText represented by a vector of
		 * RingZnElements.
		 * @return the cipher text, RingZnElement form.
		 */
		std::vector<libalgebra::RingZnElement> computeCipher(const std::vector<libalgebra::RingZnElement>& plainText) const ;

		/**
		 * Computes the plaintext associated with the ciphertext using
		 * RingZnElements.
		 * 
		 * @see computeCipher
		 */
		std::vector<libalgebra::RingZnElement> computeDecipher(const std::vector<libalgebra::RingZnElement>& cipherText) const;
		
	};	
	
}//namespace LibClassicalCrypto

#endif //__LIB_CLASSICAL_CRYPTO_CAESAR_ENGINE_H

