/***************************************************************************
 *   Copyright (C) 2007 by apterium   *
 *   apterium@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef __LIB_NUMBER_THEORY_BRUTEFORCE_COMPOSITENESS_TEST_H
#define __LIB_NUMBER_THEORY_BRUTEFORCE_COMPOSITENESS_TEST_H

#include <gmpxx.h>
#include "CompositenessTest.h"

namespace libNumberTheory
{
	/**
	 * Bruteforce compositeness test. That is, try to divide the number by
	 * every integer until its square root is found. Note that it is very
	 * similar to the bruteforce factoring algorithm.
	 * 
	 * @author apterium
	 */
	class BruteforceCompositenessTest : public CompositenessTest
	{
	public:
		/**
		 * Constructor.
		 * 
		 * @param nbr the number to test.
		 */
		BruteforceCompositenessTest(const mpz_class& nbr);
		
		/**
		 * The destructor.
		 * 
		 */
		~BruteforceCompositenessTest();
		
		/**
		 * @see CompositenessTest
		 * 
		 */
		int isComposite();

	private:
		mpz_class *number;
		
		
	};
	
}

#endif //__LIB_NUMBER_THEORY_BRUTEFORCE_COMPOSITENESS_TEST_H
