/***************************************************************************
 *   Copyright (C) 2007 by apterium   *
 *   apterium@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <iostream>
#include <math.h>
#include "Eratosthene.h"

namespace libNumberTheory
{
	Eratosthene::Eratosthene(const mpz_class& b)
	{
		this->bound = (size_type) b.get_ui();
		this->maxIndex = (size_type) (sqrt(this->bound) + 1);
#ifdef DEBUG
		std::cout << "Line " << __LINE__ << " of " << __FILE__ << std::endl;
		std::cout << "Bound mpz_class : " << b << " ; " << "bound size_type : " << this->bound << std::endl;
#endif
		this->vect = new std::vector<bool>(this->bound + 1, true);

		this->current = 1;
	}
	
	
	Eratosthene::~Eratosthene()
	{
		delete this->vect;
	}
	
	bool Eratosthene::isOver()
	{
		size_type i = this->current + 1;
		while (i < this->bound) {
			if (this->vect->at(i) == true) {
				return false;
			}
			i++;
		}
		
		return true;
	}
	
	mpz_class Eratosthene::getNextPrime()
	{
		//find the next prime index 
		this->current += 1;
		while (this->current < this->bound && this->vect->at(current) == false) {
			this->current += 1;
		}
		
		//sieve if necessary
		if (this->current < this->maxIndex) {
			for (size_type i = 2 * (this->current) ; i < this->bound ; i += this->current) {
				this->vect->at(i) = false;
			}
		}
		return this->current;
	}
	
}
