/***************************************************************************
 *   Copyright (C) 2007 by apterium   *
 *   apterium@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <iostream>
#include "libNumberTheoryTest.h"

using namespace libNumberTheory;

int errorCode = 0;

int main(int argc, char** argv)
{
	std::cout << "Running libNumberTheory tests..." << std::endl;
	bool error = false;
	
	error = error || testBruteforceCompositenessTest(mpz_class("123456789123456789123456789", 10));
	error = error || testGMPCompositenessTest(mpz_class("123456789123456789123456789", 10));
	error = error || testEratostheneSieve(mpz_class("4000000"));

	std::cout << "Done. " << std::endl;
	if (error) {
		return errorCode;
	}
	else {
		std::cout << "All tests were ok." << std::endl;
		return 0;
	}
	
		
}

int testBruteforceCompositenessTest(const mpz_class& n)
{
	errorCode++;
	std::cout << "Testing bruteforce compositeness test..." << std::endl;
	
	//TODO
	
	std::cout << "Done (bruteforce compositeness test). " << std::endl << std::endl;
	return 0;
}

int testGMPCompositenessTest(const mpz_class& n)
{
	errorCode++;
	std::cout << "Testing gmp compositeness test..." << std::endl;
	
	std::cout << "Is " << n << " composite ? " << std::endl;
	
	GMPCompositenessTest test = GMPCompositenessTest(n);
	
	int result = test.isComposite();
	std::cout << "Answer : " << result << std::endl;

	switch (result) {
	case 0:
		std::cout << "Definitely composite." << std::endl;
		break;
	case 1:
		std::cout << "Probably prime." << std::endl;
		break;
	case 2:
		std::cout << "Definitely Prime." << std::endl;
		break;
	default:
		std::cout << std::endl << "Line " << __LINE__ << " of " << __FILE__  << std::endl;
		std::cout << "Unreachable point reached. " << std::endl;
		return errorCode;
	}
	
	std::cout << "Done (gmp compositeness test). " << std::endl << std::endl;
	return 0;
}

int testEratostheneSieve(const mpz_class& n)
{
	errorCode++;
	std::cout << "Testing the Eratosthene sieve... " << std::endl;
	
	Eratosthene engine = Eratosthene(n);
	
	while (!(engine.isOver()) ) {
		std::cout << engine.getNextPrime() << std::endl;
	}
	
	std::cout << "Done (Eratosthene sieve test). " << std::endl << std::endl;
	return 0;
}
