/***************************************************************************
 *   Copyright (C) 2007 by apterium   *
 *   apterium@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "ExtensionFieldElement.h"
#include "GenericAlgorithms.h"

namespace libalgebra
{

	template<class T, class R>
	ExtensionFieldElement<T, R>::ExtensionFieldElement(const DensePolynomial<T, R>& poly,
							   const ExtensionField<T, R>* f)
	{
		this->field = f;
		this->polynomial = new DensePolynomial<T, R>(poly % (this->field->getPolynomial()));
	}
	
	template<class T, class R>
	ExtensionFieldElement<T, R>::ExtensionFieldElement(const ExtensionFieldElement<T, R>& element)
		: RingElement(element)
	{
		this->field = element.field;
		this->polynomial = new DensePolynomial<T, R>(*(element.polynomial));
	}
							       
	
	template<class T, class R>
	ExtensionFieldElement<T, R>::~ExtensionFieldElement()
	{
		delete this->polynomial;
	}	

	template<class T, class R>
	ExtensionFieldElement<T, R> ExtensionFieldElement<T, R>::operator+(const ExtensionFieldElement<T, R>& right) const
	{
		return ExtensionFieldElement<T, R>(*(this->polynomial) + *(right.polynomial), this->field);
	}

	template<class T, class R>
	ExtensionFieldElement<T, R> ExtensionFieldElement<T, R>::operator-(const ExtensionFieldElement<T, R>& right) const
	{
		return ExtensionFieldElement<T, R>(*(this->polynomial) - *(right.polynomial), this->field);
	}
	
	template<class T, class R>
	ExtensionFieldElement<T, R> ExtensionFieldElement<T, R>::operator*(const ExtensionFieldElement<T, R>& right) const
	{
		return ExtensionFieldElement<T, R>( (*(this->polynomial) * *(right.polynomial)) % (field->getPolynomial() ) , this->field);
	}

	template<class T, class R>
	ExtensionFieldElement<T, R> ExtensionFieldElement<T, R>::operator/(const ExtensionFieldElement<T, R>& divisor) const
		throw(NonInvertibleElementException<T>)
	{
		return (*this) * (divisor.inverse());
	}

	template<class T, class R>
	ExtensionFieldElement<T, R> ExtensionFieldElement<T, R>::inverse() const
		throw(NonIrreductiblePolynomialException,
		      NonInvertibleElementException<ExtensionFieldElement<T, R> >)
	{
		if (!(this->polynomial->isInvertible()) ) {
			throw NonInvertibleElementException<
			ExtensionFieldElement<T, R> >(*this);
		}
		try {
			DensePolynomial<T, R> ret = GenericAlgorithms<DensePolynomial<T, R> >
				::ModularInversion(*(this->polynomial), field->getPolynomial());
			return ExtensionFieldElement<T, R>(ret, this->field);
		} 
		catch (NonCoprimeElementsException& e) {
			if ( *(this->polynomial) != DensePolynomial<T, R>(T(0))){
				throw NonIrreductiblePolynomialException();
			}
			else {//this element is 0 thus is not invertible
				throw NonInvertibleElementException<
				ExtensionFieldElement<T, R> >(*this);
			}
		}
	}
	
	template<class T, class R>
	bool ExtensionFieldElement<T, R>::isInvertible() const
	{
		try {
			ExtensionFieldElement<T, R> inverse = this->inverse();
		}
		catch(NonCoprimeElementsException& e) {
			return false;
		}
		catch(NonInvertibleElementException<
		      ExtensionFieldElement<T, R> >& ep) {
			return false;
		}
		return true;
	}
	
	template<class U, class RR>
	std::ostream& operator<<(std::ostream& out, 
					const ExtensionFieldElement<U, RR> element)
	{
		return (out << *(element.polynomial));
	}
	
	
	
} //namespace libalgebra
