/***************************************************************************
 *   Copyright (C) 2007 by apterium   *
 *   apterium@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef __LIB_ALGEBRA_FIELD_H
#define __LIB_ALGEBRA_FIELD_H

#include <gmpxx.h>
#include "NonIntegralDomainException.h"

namespace libalgebra
{
  /**
   * Basic prototype for a (non commutative) Field. 
   *
   * @author apterium
   */
  class Field 
  {
    
  public:
	  
	  virtual ~Field()
	  {
	  };
	  
	  /**
	   * Returns this Field characteristic. 
	   *
	   * @return the characteristic of this Ring, which is a prime
	   * number if this Ring is an integral domain, as used by GMP, 
	   */
	  
	  virtual mpz_class characteristic() const = 0;
	  
	  /**
	   * Gets this Field's number of elements. 0 means this Ring is
	   * infinite, like Z, Q, polynomials, etc.
	   *
	   * @return a mpz_class representing the cardinality of this Ring.
	   */
	  virtual mpz_class cardinality() const = 0;
  };



}

#endif //__LIB_ALGEBRA_FIELD_H
