/***************************************************************************
 *   Copyright (C) 2007 by apterium   *
 *   apterium@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

namespace libalgebra
{
	template<class T>
	T GenericAlgorithms<T>::gcd(const T& t1, const T& t2)
	{
		return recursiveEuclideanAlgorithm(t1, t2);		
	}
	
	
	
	
	template<class T>
	std::vector<T> GenericAlgorithms<T>::BezoutCoefficients(const T& value, 
					       const T& modulus)
	{
		return recursiveExtendedEuclideanAlgorithm(value, modulus);
	}
	
	template<class T>
	T GenericAlgorithms<T>::ModularInversion(const T& value, const T& modulus) 
		throw(NonCoprimeElementsException)
	{
		std::vector<T> bezCoeffs = BezoutCoefficients(value, modulus);
		if ( !(bezCoeffs.at(0).isInvertible()) ) {
			throw NonCoprimeElementsException();
		}
		return bezCoeffs.at(1);
	}
	
	template<class T>
	std::vector<T> GenericAlgorithms<T>::recursiveExtendedEuclideanAlgorithm(const T& value, 
								     const T& modulus)
	{
		if (modulus == T(mpz_class(0))) {
			std::vector<T> ret;
			ret.push_back(value);
 			ret.push_back(T(mpz_class(1)) );
 			ret.push_back(T(mpz_class(0)) );
			return ret;
		}
		std::vector<T> ret = recursiveExtendedEuclideanAlgorithm(modulus, value % modulus);
		T xp = T(ret.at(1));
		T yp = T(ret.at(2));
		ret.at(1) = yp;
		ret.at(2) = xp - (value / modulus) * yp;
		return ret;
	}

	template<class T>
	T GenericAlgorithms<T>::recursiveEuclideanAlgorithm(const T& t1, 
							    const T& t2)
	{
		if (t2 == T(0) ) {
			return t2;
		}
		return recursiveEuclideanAlgorithm(t2, t1 % t2);
	}

	
} //namespace libalgebra
