/***************************************************************************
 *   Copyright (C) 2007 by apterium   *
 *   apterium@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef __LIB_ALGEBRA_RING_ELEMENT_H
#define __LIB_ALGEBRA_RING_ELEMENT_H

#include "Ring.h"

namespace libalgebra
{
	/**
	 * @short API for Ring Elements. 
	 *
	 * That is, every Ring Element whatever its type should inherit from
	 * this interface, and provide it own Ring operations operators. 
	 * Note that the operation / (division) shall give the quotient in the
	 * Euclidean division if the ring is euclidean. 
	 * Note that a constructor of the Ring Elements shall be of type (const
	 * mpz_class& value, const Ring* r) where r is of the convenient Ring type.
	 *  
	 * @author apterium
	 */ 
	class RingElement
	{
	public:
		virtual ~RingElement()
		{
		};
		
		/**
		 * Gets the ring (field) this element belongs to. Not useful !
		 * 
		 * @return a constant reference to the ring of this element.
		 */
		virtual const Ring* getRing() const = 0;
		
		/**
		 * Tells whether this element is invertible or not in the Ring.
		 * 
		 * 
		 */
		virtual bool isInvertible() const = 0;
	};	
}


#endif //__LIB_ALGEBRA_RING_ELEMENT_H
