/***************************************************************************
 *   Copyright (C) 2007 by apterium   *
 *   apterium@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef __LIB_ALGEBRA_RING_Z_ELEMENT_H
#define __LIB_ALGEBRA_RING_Z_ELEMENT_H

#include <gmpxx.h>
#include <iostream>
#include "RingElement.h"
#include "RingZ.h"


namespace libalgebra
{
	/**
	 * @short Implementation of integers, using GMP.
	 *
	 * Basic wrapper around GMP in order to implement the Ring Element
	 * interface. The operators are directly mapped to GMP operators.
	 *
	 * @author apterium
	 */
	class RingZElement : public RingElement
	{
		/**
		 * Creates a new RingZElement, given its value.
		 * 
		 * @param k a constant reference to the value of the desired
		 * integer.
		 * @param r a reference to the RingZ which is shared by everyone.
		 */
		RingZElement(const mpz_class& k, RingZ& r);
		
		/**
		 * Copies the value, pointer to the ring.
		 * 
		 */
		RingZElement(const RingZElement& copy);
		
		/**
		 * Destructor.
		 * 
		 */
		~RingZElement();
		
		/**
		 * @see RingElement
		 * 
		 */
		inline const RingZ* getRing() const
		{
			return this->ring;
		};
		
		inline bool isInvertible() const
		{
			return (*(this->value) == 1 or *(this->value) == -1);
		};
		
		inline RingZElement operator+(const RingZElement& right)
		{
			return RingZElement(*(right.value) + *(this->value), *(this->ring));
		};
		
		inline RingZElement operator-(const RingZElement& right)
		{
			return RingZElement(*(right.value) - *(this->value), *(this->ring));
		};

		inline RingZElement operator/(const RingZElement& right)
		{
			return RingZElement(*(right.value) / *(this->value), *(this->ring));
		};

		inline RingZElement operator%(const RingZElement& right)
		{
			return RingZElement(*(right.value) % *(this->value), *(this->ring));
		};

		inline RingZElement operator*(const RingZElement& right)
		{
			return RingZElement(*(right.value) * *(this->value), *(this->ring));
		};
		
	private:
		mpz_class* value;
		RingZ* ring;
		
		inline friend bool operator<<(std::ostream& out, const RingZElement& integer)
		{
			return (out << (*(integer.value)));
		};
		
		inline friend bool operator==(const RingZElement& left, const RingZElement& right)
		{
			return (*(left.value) == *(right.value));
		};
	};
	
} //namespace libalgebra

#endif //__LIB_ALGEBRA_RING_Z_ELEMENT_H
