/***************************************************************************
 *   Copyright (C) 2007 by apterium   *
 *   apterium@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <iostream>
#include <vector>
#include <map>
#include <gmp.h>
#include "libalgebratest.h"


using namespace libalgebra;

/**
 * A global variable that records the reason of the exit. 
 */
int errorCode = 0;

namespace 
{
	const RingZn ring = RingZn(mpz_class(26) );

	std::vector<RingZnElement> initialize()
	{
		std::vector<RingZnElement> coe = 
			std::vector<RingZnElement>();
		coe.push_back(RingZnElement(mpz_class(1), &ring));
		coe.push_back(RingZnElement(mpz_class(3), &ring));
		coe.push_back(RingZnElement(mpz_class(0), &ring));
		coe.push_back(RingZnElement(mpz_class(3), &ring));
		return coe;
	}
	std::vector<RingZnElement> coeffs 
	= initialize();	

	DensePolynomial<RingZnElement, RingZn> poly
	= DensePolynomial<RingZnElement, RingZn>(coeffs, &ring);

	
	ExtensionField<RingZnElement, RingZn> field = 
		ExtensionField<RingZnElement, RingZn>(poly);
}


int main(int argc, char** argv)
{ 
	std::cout << "Running Lib Algebra tests..." << std::endl ;
	bool error = false;
	error = error || testZn(mpz_class(20));
	error = error || testRingZnElement(mpz_class(14), 
					   mpz_class(13));
	std::vector<RingZnElement > coeffs1;
	coeffs1.reserve(5);
	std::cout << "coeffs1 size : " << coeffs1.size() << std::endl;
	coeffs1.push_back(RingZnElement(mpz_class(1), &ring));
	coeffs1.push_back(RingZnElement(mpz_class(3), &ring));
	coeffs1.push_back(RingZnElement(mpz_class(0), &ring));
	
	std::cout << "coeffs1 size : " << coeffs1.size() << std::endl;
	DensePolynomial<RingZnElement, RingZn > poly1(coeffs1, &ring);
	
	coeffs1.push_back(RingZnElement(mpz_class(3), &ring));
	DensePolynomial<RingZnElement, RingZn > poly2(coeffs1, &ring);
	
	error = error || testDensePolynomial(poly1, poly2, 
					     RingZnElement(mpz_class(13), &ring));
	

	DenseMatrix<RingZnElement, RingZn> mat1(5, 5, &ring);
	DenseMatrix<RingZnElement, RingZn > mat2(5, 5, &ring);
	error = error || testDenseMatrix(mat1, mat2);
	

	//	error = error || testExtensionField(poly2);
	//	ExtensionField<RingZnElement<&ring> > 
	//		field = ExtensionField<RingZnElement<&ring> > (poly2) ;
	//	NonCoprimeElementsException exc = NonCoprimeElementsException();
	//	ExtensionFieldElement<RingZnElement, &field> element(poly1);
	//	std::cout << element<< std::endl;
	//	error = error || testExtensionFieldElement();

	
	std::cout << "Done. "<< std::endl;
	if (!error) {
		std::cout << "All tests are ok." << std::endl ;
	}

	return error;
}

int testZn(const mpz_class& n)
{
	errorCode++;
	std::cout << "Testing the implementation of Z/nZ..." << std::endl;
	//  RingZn zn(n);
	std::map<mpz_class, mpz_class> nb;
	nb[mpz_class(2)] = mpz_class(2);
	nb[mpz_class(5)] = mpz_class(1);
	RingZn zn(nb);
	
	
	std::cout << "Testing its cardinality..." << std::endl;
	if (zn.cardinality() != n) {
		return errorCode;
	} 
	
	std::cout << "Testing whether it is an integral domain..." << std::endl;
	
	std::cout << "Ok." << std::endl;
	return 0;
}

int testRingZnElement(const mpz_class &val, 
		      const mpz_class &ops)
{
	errorCode++;
	std::cout << "Testing the implementation of RingZnElement... " << std::endl;
	RingZnElement elt(val, &ring);
	std::cout << "Value of val : " << elt.getValue() << std::endl;

	std::cout << "val : "<< elt << std::endl;
	
	RingZnElement operand(ops, &ring);
	std::cout << "Value of the operand : " << operand.getValue() << std::endl;
	
	std::cout << "The ring they belong to cardinality : " << elt.getRing()->cardinality() << std::endl;
	
	const RingZn* r = elt.getRing(); 
	std::cout << "An other way for it..." << r->cardinality() << std::endl;
	std::cout << "Now ? " << elt.getRing()->cardinality() << std::endl;

	
	RingZnElement resultPlus = elt + operand;
	std::cout << "Value of the addition : " << resultPlus.getValue() << std::endl;
	
	RingZnElement resultMinus = elt - operand;
	std::cout << "Value of the subtraction : " << resultMinus.getValue() << std::endl;
	
	RingZnElement resultMult = elt * operand;
	std::cout << "Value of the multiplication : " << resultMult.getValue() << std::endl;
	
	try {
		RingZnElement resultInvVal = elt.inverse();
		std::cout << "Inverse of val : " << resultInvVal.getValue() << std::endl;
	} catch (const NonInvertibleElementException<RingZnElement>& e) {
		std::cout << "elt is not invertible." << std::endl;
	} catch (...) {
		std::cerr << "Unknown exception ! " << std::endl 
			  << "Line __LINE__ of __FILE__ " << std::endl;
	}
	
	try {
		RingZnElement resultInvOperand = operand.inverse() ;
		std::cout << "Inverse of operand : " << resultInvOperand.getValue() << std::endl;
	} catch (const NonInvertibleElementException<RingZnElement>& e) {
		std::cout << "operand is not invertible." << std::endl;
	} catch (...) {
		std::cerr << "Unknown exception ! " << std::endl 
			  << "Line __LINE__ of __FILE__ " << std::endl;
	}
	
	try {
		RingZnElement resultDiv = elt / operand;
		std::cout << "Value of the division : " << resultDiv.getValue() << std::endl;
	} catch (const NonInvertibleElementException<RingZnElement>& e) {
		std::cout << "operand is not invertible." << std::endl;
	} catch (...) {
		std::cerr << "Unknown exception ! " << std::endl 
			  << "Line __LINE__ of __FILE__ " << std::endl;
	} 
	return 0;
}

int testDensePolynomial(DensePolynomial<RingZnElement, RingZn>& poly1, 
			const DensePolynomial<RingZnElement, RingZn>& poly2, 
		     const RingZnElement& value)
{
	errorCode++;
	std::cout << "Testing the implementation of Dense Polynomials..." << std::endl;
	
	std::cout << "Degree of poly1 : " << std::endl;
	std::cout << poly1.degree() << std::endl;
	
	std::cout << "poly1 : " << std::endl;
	std::cout << poly1 << std::endl;
	
	std::cout << "Simplification of poly1 : " << std::endl;
	poly1.reduce();
	std:: cout << poly1 << std::endl;
	std::cout << "poly2 : " << poly2 << std::endl;
	std::cout << "poly1 + poly2 : " << poly1 + poly2 << std::endl;
	std::cout << "poly2 + poly1 : " << poly2 + poly1 << std::endl;
	std::cout << "poly1 - poly2 : " << poly1 - poly2 << std::endl;
	std::cout << "poly2 - poly1 : " << poly2 - poly1 << std::endl;
	std::cout << "poly1 * poly2 : " << poly1 * poly2 << std::endl;
	std::cout << "poly2 * poly1 : " << poly2 * poly1 << std::endl;
	std::cout << "poly2 / poly1 : " << poly2 / poly1 << std::endl;
	std::cout << "poly1 / poly2 : " << poly1 / poly2 << std::endl;
	std::cout << "poly2 % poly1 : " << poly2 % poly1 << std::endl;
	std::cout << "poly1 % poly2 : " << poly1 % poly2 << std::endl;
	
	std::cout << "value : " << value << std::endl;
	std::cout << "poly1(value) : " << poly1.eval(value) << std::endl;
	std::cout << "poly2(value) : " << poly2.eval(value) << std::endl;
	
	std::cout << "Done." << std::endl;
	return 0;
}

int testDenseMatrix(const DenseMatrix<RingZnElement, RingZn>& mat1, 
		    const DenseMatrix<RingZnElement, RingZn>& mat2)
{
	errorCode++;
	
	std::cout << "mat1 : " << mat1 << std::endl;
	std::cout << "mat2 : " << mat2 << std::endl;
	
	std::cout << "mat1 + mat2 : " << mat1 + mat2 << std::endl;
	std::cout << "mat1 - mat2 : " << mat1 - mat2 << std::endl;
	std::cout << "mat1 * mat2 : " << mat1 * mat2 << std::endl;

	//	std::cout << "Cofactor 1, 1, of mat1 : " << mat1.cofactorSubMatrix(1, 1) << std::endl;

	std::cout << "Determinant of mat1 : " << mat1.determinant() << std::endl;
	
	std::cout << "More seriously... " << std::endl;

	DenseMatrix<RingZnElement, RingZn> id = DenseMatrix<RingZnElement, RingZn>::identity(2, &ring);
	std::cout << "The identity of rank 2 and its determinant : " << id  << std::endl << id.determinant() << std::endl;
	
	std::vector<std::vector<RingZnElement> > lines;
	std::vector<RingZnElement> line1;
	std::vector<RingZnElement> line2;
	line1.push_back(RingZnElement(2, &ring) );
	line1.push_back(RingZnElement(13, &ring) );
	line2.push_back(RingZnElement(13, &ring) );
	line2.push_back(RingZnElement(2, &ring) );
	
	lines.push_back(line1);
	lines.push_back(line2);
	
	DenseMatrix<RingZnElement, RingZn> matrix(lines, &ring);
	
	std::cout << "The problematic matrix : " << matrix * id << std::endl;
	std::cout << "Its determinant : " << matrix.determinant() << std::endl;
	std::cout << "Its square : matrix * matrix = " << matrix + matrix << std::endl;
	std::cout << "3 * matrix = " << RingZnElement(3, &ring) * matrix << std::endl;
	DenseMatrix<RingZnElement, RingZn> inverse = matrix.inverse();
	std::cout << "Matrix inverse : " << inverse << std::endl;
	
	std::cout << "The product gives back the identity : " << inverse * matrix << std::endl;
	//	std::cout << "Really ? " << (inverse * matrix == id) << std::endl;
	return 0;
}

int testExtensionField(const DensePolynomial<RingZnElement, RingZn>& poly)
{
	errorCode++;

	std::cout << "Testing the implementation of Extension Fields..." << std::endl;
	ExtensionField<RingZnElement, RingZn> 
		field = ExtensionField<RingZnElement, RingZn> (poly) ;
	std::cout << "The extension field polynomial : " << field.getPolynomial() << std::endl;
	std::cout << "The extension field characteristic : " << field.characteristic() << std::endl;
	std::cout << "Its cardinality : " << field.cardinality() << std::endl;
	return 0;
}


int testExtensionFieldElement()
{
	
	errorCode++;
	std::cout << "Testing the implementation of Extension Field Elements..." << std::endl;
	//ExtensionFieldElement<RingZnElement<&ring>, &field> element;
	
	return 0;
}
