/***************************************************************************
 *   Copyright (C) 2007 by apterium   *
 *   apterium@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef __LIBALGEBRATEST_H
#define __LIBALGEBRATEST_H

#include <gmpxx.h>
#include "libalgebra.h"

using namespace libalgebra;

/**
 * Runs all tests on the whole libalgebra library. 
 * 
 *
 * @author apterium
 * @todo Provide arguments in order to tell the implementation which
 * tests should be ran, when the library will become bigger and
 * bigger. 
 * @return the error code of the failed test, or 0 if no.
 */
int main(int argc, char** argv);

/**
 * Runs tests on the implementation of the ring Zn.
 *
 * @param n a GMP integer.
 */
int testZn(const mpz_class &n);

/**
 * Test the implementation of RingZnElement.
 *
 * @param n the modulus
 * @param val the value we want to test.
 * @param ops the value of an other RingZnElement for operator testing.
 */
int testRingZnElement(const mpz_class &val, 
		      const mpz_class &ops);


/**
 * Tests the implementation of dense polynomials.
 *
 * @param poly1 the first polynomial.
 * @param poly2 the second polynomial
 * @param value the value where both polynomials shall be evaluated
 * @return the error code
 */
int testDensePolynomial(DensePolynomial<RingZnElement, RingZn>& poly1, 
			const DensePolynomial<RingZnElement, RingZn>& poly2, 
		     const RingZnElement& value);
/**
 * Tests the implementation of dense matrices.
 *
 *
 */ 
int testDenseMatrix(const DenseMatrix<RingZnElement, RingZn>& mat1, 
		    const DenseMatrix<RingZnElement, RingZn>& mat2);

int testExtensionField(const DensePolynomial<RingZnElement, RingZn>& poly1);

int testExtensionFieldElement();

#endif //__LIBALGEBRATEST_H
