/***************************************************************************
 *   Copyright (C) 2007 by Andrea Moro   *
 *   andrea_moro@email.it   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 ***************************************************************************/

#include <QtGui>
#include <QAssistantClient>
#include <QFile>
#include "mainWindow.h"
#include "mdichild.h"
#include "drawParam.h"
#include "startOpt.h"
#include "analysisOpt.h"
#include "furtOpt.h"
#include "textOpt.h"
#include "langOpt.h"
#include "CaesarRot13.h"
#include "vigenere.h"
#include "hill.h"
#include "atbash.h"
#include "playfair.h"
#include "ADFGVX.h"
#include "byteAdd.h"
#include "byteXor.h"
#include "vernam.h"
#include "Factorisation.h"
#include "PrimeNumberTest.h"
#include "CaesarAnalysis.h"

mainWindow::mainWindow(QWidget * parent)
{
	setupUi(this);
	initializeAssistant();
	qsrand(QTime(0,0,0).secsTo(QTime::currentTime()));
	workspace = new QMdiArea(this);
	setCentralWidget(workspace);
	createToolBars();
	createConnections();
	updateMenu();
	opttext_changed = new QAction(this);
	if(!loadOptions())
	{
		/*set default options*/
		options.plot_opt.bar_width=1;
		options.plot_opt.curve_width=1;
		options.plot_opt.zero=false;
		options.plot_opt.autocorr_relative=false;
		options.analysis_opt.caesar_derived_key=true;
		options.analysis_opt.caesar_correlation=true;
		options.analysis_opt.caesar_histograms=true;
		options.analysis_opt.vxa_derived_key=true;
		options.analysis_opt.vxa_derived_key_len=true;
		options.analysis_opt.vxa_autocorrelation=true;
		options.analysis_opt.vxa_base_analysis=false;
		options.analysis_opt.num_bytes_entropy=128;
		options.analysis_opt.cleartext_alph=0;
		options.text_opt.char_unchanged=true;
		options.text_opt.keep_case=true;
		options.text_opt.distinguish_case=false;
		options.text_opt.alphabet="ABCDEFGHIJKLMNOPQRSTUVWXYZ";
		options.text_opt.stat_file="";
		options.text_opt.separate=true;
		options.text_opt.separator='X';
		options.start_opt.sample_text=true;
		options.further_opt.intro=false;
		options.further_opt.alice_name="Alice";
		statusBar()->showMessage(tr("Ready"),10000);
	}
	if(options.start_opt.sample_text)
	{
		action_New->activate(QAction::Trigger);
		currentMdiChild()->setPlainText(tr("Welcome to CrypToolinux"));
	}
}
bool mainWindow::loadOptions()
{
	if(!QFile::exists(QString("~/.cryptoolinux/options")))
		return false;
	return false;
}
void mainWindow::saveOptions()
{
	QDir ctlDir((QDir::home().absolutePath())+QString("/.cryptoolinux/"));
	if(!ctlDir.exists())
	{
		QDir tmp(QDir::home());
		tmp.mkdir(QString("/.cryptoolinux/"));
	}
	qApp->closeAllWindows();
}
void mainWindow::initializeAssistant()
{
	QString locale = QLocale::system().name();
	locale.truncate(2);
	if(QFile::exists(QString("/usr/share/cryptoolinux/help/")+locale+QString("/help.adp")))
		helpPath=QString("/usr/share/cryptoolinux/help/"+locale);
	else if(QFile::exists(QString("../doc/help/")+locale+QString("/help.adp")))
		helpPath=QString("../doc/help/"+locale);
	else if(QFile::exists(QString("/usr/share/cryptoolinux/help/en/help.adp")))
		helpPath=QString("/usr/share/cryptoolinux/help/en/");
	else if(QFile::exists(QString("../doc/help/en/help.adp")))
		helpPath=QString("../doc/help/en/");
	else
	{
		action_Help->setEnabled(false);
		helpPath="";
		return;
	}
	QStringList arguments;
	arguments << "-profile" << helpPath+QString("/help.adp");
	assistantClient = new QAssistantClient(
			QLibraryInfo::location(QLibraryInfo::BinariesPath), this);
	assistantClient->setArguments(arguments);
}
void mainWindow::assistant()
{
	assistantClient->showPage(helpPath+QString("/cryptoolinux.html"));
}
void mainWindow::newFile()
{
	MdiChild *child = createMdiChild();
	child->newFile();
	child->show();
}
void mainWindow::open()
{
	QString fileName = QFileDialog::getOpenFileName(this);
	if (!fileName.isEmpty())
	{
		QMdiSubWindow *existing = findMdiChild(fileName);
		if (existing)
		{
			workspace->setActiveSubWindow(existing);
			return;
		}

		MdiChild *child = createMdiChild();
		if (child->loadFile(fileName))
		{
			statusBar()->showMessage(tr("File loaded"), 2000);
			child->show();
		}
		else
			child->close();
	}
}
void mainWindow::save()
{
	if (currentMdiChild()->save())
		statusBar()->showMessage(tr("File saved"), 2000);
}
void mainWindow::saveAs()
{
	if (currentMdiChild()->saveAs())
		statusBar()->showMessage(tr("File saved"), 2000);
}
void mainWindow::undo()
{
	currentMdiChild()->undo();
}
void mainWindow::redo()
{
	currentMdiChild()->redo();
}
void mainWindow::cut()
{
	currentMdiChild()->cut();
}
void mainWindow::copy()
{
	currentMdiChild()->copy();
}
void mainWindow::paste()
{
	currentMdiChild()->paste();
}
void mainWindow::selectAll()
{
	currentMdiChild()->selectAll();
}
void mainWindow::toolbar()
{
	if(action_Toolbar->isChecked())
	{
		fileToolBar->setVisible(true);
		editToolBar->setVisible(true);
	}
	else
	{
		fileToolBar->setVisible(false);
		editToolBar->setVisible(false);
	}
}
void mainWindow::statusbar()
{
	if(action_Status_Bar->isChecked())
		statusBar()->setVisible(true);
	else
		statusBar()->setVisible(false);
}
void mainWindow::about()
{
	QMessageBox::about(this, tr("About CrypTooLinux ")+QString(VERSION),
		tr("<b>CrypTooLinux</b> is a clone of <b>CrypTool</b> on linux<br>"
			"<a href=\"http://www.cryptoolinux.net\">"
			"http://www.cryptoolinux.net</a><br>"
			"<a href=\"http://www.cryptool.com\">http://www.cryptool.com</a><br>"));
}
MdiChild* mainWindow::createMdiChild()
{
	QMdiSubWindow *subWin = new QMdiSubWindow;
	MdiChild *child = new MdiChild;
	subWin->setWidget(child);
	subWin->setAttribute(Qt::WA_DeleteOnClose);
	workspace->addSubWindow(subWin);
	connect(child, SIGNAL(copyAvailable(bool)),actionCu_t, SLOT(setEnabled(bool)));
	connect(child, SIGNAL(copyAvailable(bool)),action_Copy, SLOT(setEnabled(bool)));
	connect(child, SIGNAL(undoAvailable(bool)),action_Undo, SLOT(setEnabled(bool)));
	connect(child, SIGNAL(redoAvailable(bool)),actionRe_do, SLOT(setEnabled(bool)));
	return child;
}
void mainWindow::setActiveSubWindow(QWidget *window)
{
	if (!window)
		return;
	workspace->setActiveSubWindow(qobject_cast<QMdiSubWindow *>(window));
}
void mainWindow::createConnections()
{
	connect(action_New, SIGNAL(triggered()), this, SLOT(newFile()));
	connect(action_Open, SIGNAL(triggered()), this, SLOT(open()));
	connect(action_Save, SIGNAL(triggered()), this, SLOT(save()));
	connect(actionSave_as, SIGNAL(triggered()), this, SLOT(saveAs()));
	connect(action_Undo, SIGNAL(triggered()), this, SLOT(undo()));
	connect(actionRe_do, SIGNAL(triggered()), this, SLOT(redo()));
	connect(actionCu_t, SIGNAL(triggered()), this, SLOT(cut()));
	connect(action_Copy, SIGNAL(triggered()), this, SLOT(copy()));
	connect(action_Paste, SIGNAL(triggered()), this, SLOT(paste()));
	connect(actionSelect_All, SIGNAL(triggered()), this, SLOT(selectAll()));
	connect(action_Toolbar, SIGNAL(triggered()), this, SLOT(toolbar()));
	connect(action_Status_Bar, SIGNAL(triggered()), this, SLOT(statusbar()));
	connect(action_Exit, SIGNAL(triggered()), this, SLOT(saveOptions()));
	connect(actionA_bout_CrypTool, SIGNAL(triggered()), this, SLOT(about()));
	connect(actionAbout_Qt, SIGNAL(triggered()), qApp, SLOT(aboutQt()));
	connect(action_Tile, SIGNAL(triggered()), workspace, SLOT(tileSubWindows()));
	connect(action_Close_All, SIGNAL(triggered()), workspace, SLOT(closeAllSubWindows()));
	connect(action_Cascade, SIGNAL(triggered()), workspace, SLOT(cascadeSubWindows()));
	connect(action_Help,SIGNAL(triggered()), this, SLOT(assistant()));
	connect(workspace,SIGNAL(subWindowActivated(QMdiSubWindow *)), this, SLOT(updateMenu()));
	connect(action_Plot_Options, SIGNAL(triggered()), this, SLOT(opt_draw()));
	connect(action_Starting_Options, SIGNAL(triggered()), this, SLOT(opt_start()));
	connect(action_Analysis_Options, SIGNAL(triggered()), this, SLOT(opt_analysis()));
	connect(action_Further_Options, SIGNAL(triggered()), this, SLOT(opt_furt()));
	connect(action_Text_Options, SIGNAL(triggered()), this, SLOT(opt_text()));
	connect(action_Language_Options, SIGNAL(triggered()), this, SLOT(opt_lang()));
/*cryptofunctions*/
	connect(actionCaesar_Rot_13, SIGNAL(triggered()), this, SLOT(crDe_caesarRot()));
	connect(action_Vigen_re, SIGNAL(triggered()), this, SLOT(crDe_vigenere()));
	connect(action_Hill, SIGNAL(triggered()), this, SLOT(crDe_hill()));
	connect(action_Substitution_Atbash, SIGNAL(triggered()), this, SLOT(crDe_atbash()));
	connect(action_Playfair, SIGNAL(triggered()), this, SLOT(crDe_playfair()));
	connect(action_ADFGVX, SIGNAL(triggered()), this, SLOT(crDe_ADFGVX()));
	connect(actionByte_A_ddition, SIGNAL(triggered()), this, SLOT(crDe_byteAdd()));
	connect(action_XOR, SIGNAL(triggered()), this, SLOT(crDe_byteXor()));
	connect(actionV_ernam, SIGNAL(triggered()), this, SLOT(crDe_Vernam()));
	connect(action_Factorisation_of_a_Number_2, SIGNAL(triggered()), this, SLOT(crFactorisation_of_a_Number_2()));
	connect(actionPrime_Number_Test, SIGNAL(triggered()), this, SLOT(cractionPrime_Number_Test()));
	connect(action_CaesarAnalysis, SIGNAL(triggered()), this, SLOT(crCaesarAnalysis()));
}
MdiChild* mainWindow::currentMdiChild()
{
	if (QMdiSubWindow *currentSubWindow = workspace->currentSubWindow())
		return qobject_cast<MdiChild *>(currentSubWindow->widget());
	return 0;
}
QMdiSubWindow * mainWindow::findMdiChild(const QString &fileName)
{
	QString canonicalFilePath = QFileInfo(fileName).canonicalFilePath();
	foreach (QMdiSubWindow *window, workspace->subWindowList())
	{
		MdiChild *mdiChild = qobject_cast<MdiChild *>(window->widget());
		if (mdiChild->currentFile() == canonicalFilePath)
			return window;
	}
	return 0;
}
void mainWindow::createToolBars()
{
	action_New->setIcon(QIcon(":/filenew.xpm"));
	action_Open->setIcon(QIcon(":/fileopen.xpm"));
	action_Save->setIcon(QIcon(":/filesave.xpm"));
	actionCu_t->setIcon(QIcon(":/editcut.xpm"));
	action_Copy->setIcon(QIcon(":/editcopy.xpm"));
	action_Paste->setIcon(QIcon(":/editpaste.xpm"));
	fileToolBar = addToolBar(tr("File"));
	fileToolBar->addAction(action_New);
	fileToolBar->addAction(action_Open);
	fileToolBar->addAction(action_Save);
	editToolBar = addToolBar(tr("Edit"));
	editToolBar->addAction(actionCu_t);
	editToolBar->addAction(action_Copy);
	editToolBar->addAction(action_Paste);
}
void mainWindow::updateMenu()
{
	if(currentMdiChild()==0)
	{
		actionRe_do->setEnabled(false);
		action_Undo->setEnabled(false);
		actionCu_t->setEnabled(false);
		action_Copy->setEnabled(false);
		actionSelect_All->setEnabled(false);
		action_Paste->setEnabled(false);
		action_Save->setEnabled(false);
		actionSave_as->setEnabled(false);
		actionCaesar_Rot_13->setEnabled(false);
		action_Vigen_re->setEnabled(false);
		action_Hill->setEnabled(false);
		action_Substitution_Atbash->setEnabled(false);
		action_Playfair->setEnabled(false);
		action_ADFGVX->setEnabled(false);
		actionByte_A_ddition->setEnabled(false);
		action_XOR->setEnabled(false);
		//actionV_ernam->setEnabled(false);
		//action_Factorisation_of_a_Number_2->setEnabled(false);
		//actionPrime_Number_Test->setEnabled(false);
	}
	else
	{
		actionSelect_All->setEnabled(true);
		action_Paste->setEnabled(true);
		action_Save->setEnabled(true);
		actionSave_as->setEnabled(true);
		actionCaesar_Rot_13->setEnabled(true);
		action_Vigen_re->setEnabled(true);
		action_Hill->setEnabled(true);
		action_Substitution_Atbash->setEnabled(true);
		action_Playfair->setEnabled(true);
		action_ADFGVX->setEnabled(true);
		actionByte_A_ddition->setEnabled(true);
		action_XOR->setEnabled(true);
		//actionV_ernam->setEnabled(true);
		//action_Factorisation_of_a_Number_2->setEnabled(true);
		//actionPrime_Number_Test->setEnabled(true);
	}
}
void mainWindow::newSubWindow(QString title, QString content)
{
	MdiChild *child = createMdiChild();
	child->newFile();
	child->setWindowTitle(title);
	child->setText(content);
	child->show();
}
void mainWindow::opt_draw()
{
	drawParam *tmp = new drawParam(this);
	tmp->show();
}
void mainWindow::opt_start()
{
	startOpt *tmp = new startOpt(this);
	tmp->show();
}
void mainWindow::opt_analysis()
{
	analysisOpt *tmp = new analysisOpt(this);
	tmp->show();
}
void mainWindow::opt_furt()
{
	furtOpt *tmp = new furtOpt(this);
	tmp->show();
}
void mainWindow::opt_text()
{
	textOpt *tmp = new textOpt(this);
	tmp->show();
}
void mainWindow::opt_lang()
{
	langOpt *tmp = new langOpt(this);
	tmp->show();
}
/******************** CLASSICAL CRYPTOFUNCTIONS ********************/
void mainWindow::crDe_caesarRot()
{
	if(currentMdiChild()->toPlainText().length()==0)
	{
		QMessageBox::warning(this, tr("Warning"),
			tr("The input must contain at least 1 character"));
		return;
	}
	CaesarRot13 *tmp = new CaesarRot13(this,currentMdiChild()->toPlainText(),
					currentMdiChild()->windowTitle());
	tmp->show();
}
void mainWindow::crDe_vigenere()
{
	if(currentMdiChild()->toPlainText().length()==0)
	{
		QMessageBox::warning(this, tr("Warning"),
			tr("The input must contain at least 1 character"));
		return;
	}
	Vigenere *tmp = new Vigenere(this,currentMdiChild()->toPlainText(),
					currentMdiChild()->windowTitle());
	tmp->show();
}
void mainWindow::crDe_hill()
{
	if(currentMdiChild()->toPlainText().length()==0)
	{
		QMessageBox::warning(this, tr("Warning"),
			tr("The input must contain at least 1 character"));
		return;
	}
	hill *tmp = new hill(this,currentMdiChild()->toPlainText(),
				currentMdiChild()->windowTitle());
	tmp->show();
}
void mainWindow::crDe_atbash()
{
	if(currentMdiChild()->toPlainText().length()==0)
	{
		QMessageBox::warning(this, tr("Warning"),
			tr("The input must contain at least 1 character"));
		return;
	}
	Atbash *tmp = new Atbash(this,currentMdiChild()->toPlainText(),
				currentMdiChild()->windowTitle());
	tmp->show();
}
void mainWindow::crDe_playfair()
{
	if(currentMdiChild()->toPlainText().length()==0)
	{
		QMessageBox::warning(this, tr("Warning"),
			tr("The input must contain at least 1 character"));
		return;
	}
	Playfair *tmp = new Playfair(
		this,currentMdiChild()->toPlainText(),currentMdiChild()->windowTitle());
	tmp->show();
}
void mainWindow::crDe_ADFGVX()
{
	if(currentMdiChild()->toPlainText().length()==0)
	{
		QMessageBox::warning(this, tr("Warning"),
			tr("The input must contain at least 1 character"));
		return;
	}
	ADFGVX *tmp = new ADFGVX(
		this,currentMdiChild()->toPlainText(),currentMdiChild()->windowTitle());
	tmp->show();
}
void mainWindow::crDe_byteAdd()
{
	if(currentMdiChild()->toPlainText().length()==0)
	{
		QMessageBox::warning(this, tr("Warning"),
			tr("The input must contain at least 1 character"));
		return;
	}
	byteAdd *tmp = new byteAdd(
		this,currentMdiChild()->toPlainText(),currentMdiChild()->windowTitle());
	tmp->show();
}
void mainWindow::crDe_byteXor()
{
	if(currentMdiChild()->toPlainText().length()==0)
	{
		QMessageBox::warning(this, tr("Warning"),
			tr("The input must contain at least 1 character"));
		return;
	}
	byteXor *tmp = new byteXor(
		this,currentMdiChild()->toPlainText(),currentMdiChild()->windowTitle());
	tmp->show();
}
void mainWindow::crDe_Vernam()
{
	if(currentMdiChild()->toPlainText().length()==0)
	{
		QMessageBox::warning(this, tr("Warning"),
			tr("The input must contain at least 1 character"));
		return;
	}
	QString fileName = QFileDialog::getOpenFileName(this);
	QString key,title;
	if(fileName.isEmpty())
		return;
	QFile file(fileName);
	if(!file.open(QIODevice::ReadOnly))
		return;
	if(currentMdiChild()->toPlainText().length()<=file.size())
		key.append(file.read(currentMdiChild()->toPlainText().length()));
	else
		key.append(file.readAll());
	Vernam *tmp = new Vernam(currentMdiChild()->toPlainText(),key);
	title.append(tr("Vernam encryption of <"));
	title.append(currentMdiChild()->windowTitle());
	title.append(tr(">, with key stream from <"));
	title.append(fileName);
	title.append(">");
	newSubWindow(title, tmp->crypt()->toAscii());
}

/*********************** INDIVIDUAL PROCEDURES FUNCTIONS **********************/
void mainWindow::cractionPrime_Number_Test()
{
	PrimeNumberTest *tmp = new PrimeNumberTest(this);
	tmp->show();
}

void mainWindow::crFactorisation_of_a_Number_2()
{
	Factorisation *tmp = new Factorisation(this);
	tmp->show();
}

/*********************** CRYPTANALYSIS FUNCTIONS *************************/
void mainWindow::crCaesarAnalysis()
{
	if(currentMdiChild()->toPlainText().length() <= 1)
	{
		QMessageBox::warning(this, tr("Warning"),
			tr("The input must contain at least 2 characters"));
		return;
	}
	CaesarAnalysis *tmp = new CaesarAnalysis(this,currentMdiChild()->toPlainText(),
					currentMdiChild()->windowTitle());
	//TODO : something.
	//	tmp->show();	
	
}


#include "mainWindow.moc"
