/***************************************************************************
 *   Copyright (C) 2007 by Andrea Moro   *
 *   andrea_moro@email.it   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 ***************************************************************************/

#include <QtGui>
#include "mdichild.h"

MdiChild::MdiChild()
{
	setAttribute(Qt::WA_DeleteOnClose);
	isUntitled = true;
}

void MdiChild::newFile()
{
	static int sequenceNumber = 1;

	isUntitled = true;
	curFile = tr("document%1.txt").arg(sequenceNumber++);
	setWindowTitle(curFile + "[*]");

	connect(document(), SIGNAL(contentsChanged()),
		this, SLOT(documentWasModified()));
}

bool MdiChild::loadFile(const QString &fileName)
{
	QFile file(fileName);
	if (!file.open(QFile::ReadOnly | QFile::Text))
	{
		QMessageBox::warning(this, "CrypTooLinux",
			tr("Cannot read file %1:\n%2.")
			.arg(fileName)
			.arg(file.errorString()));
		return false;
	}

	QTextStream in(&file);
	QApplication::setOverrideCursor(Qt::WaitCursor);
	setPlainText(in.readAll());
	QApplication::restoreOverrideCursor();

	setCurrentFile(fileName);

	connect(document(), SIGNAL(contentsChanged()),
		this, SLOT(documentWasModified()));

	return true;
}

bool MdiChild::save()
{
	if (isUntitled)
		return saveAs();
	else
		return saveFile(curFile);
}

bool MdiChild::saveAs()
{
	QString fileName = QFileDialog::getSaveFileName(this,
				tr("Save As"),curFile);
	if (fileName.isEmpty())
		return false;
	return saveFile(fileName);
}

bool MdiChild::saveFile(const QString &fileName)
{
	QFile file(fileName);
	if (!file.open(QFile::WriteOnly | QFile::Text))
	{
		QMessageBox::warning(this, "CrypTooLinux",
			tr("Cannot write file %1:\n%2.")
			.arg(fileName)
			.arg(file.errorString()));
		return false;
	}

	QTextStream out(&file);
	QApplication::setOverrideCursor(Qt::WaitCursor);
	out << toPlainText();
	QApplication::restoreOverrideCursor();

	setCurrentFile(fileName);
	return true;
}

QString MdiChild::userFriendlyCurrentFile()
{
	return strippedName(curFile);
}

void MdiChild::closeEvent(QCloseEvent *event)
{
	if (maybeSave())
		event->accept();
	else
		event->ignore();
}

void MdiChild::documentWasModified()
{
	setWindowModified(document()->isModified());
}

bool MdiChild::maybeSave()
{
	if (document()->isModified())
	{
		QMessageBox::StandardButton ret;
		ret = QMessageBox::warning(this, "CrypTooLinux",
			tr("'%1' has been modified.\n"
				"Do you want to save your changes?")
			.arg(userFriendlyCurrentFile()),
			QMessageBox::Save | QMessageBox::Discard
			| QMessageBox::Cancel);
		if (ret == QMessageBox::Save)
			return save();
		else if (ret == QMessageBox::Cancel)
			return false;
	}
	return true;
}

void MdiChild::setCurrentFile(const QString &fileName)
{
	curFile = QFileInfo(fileName).canonicalFilePath();
	isUntitled = false;
	document()->setModified(false);
	setWindowModified(false);
	setWindowTitle(userFriendlyCurrentFile() + "[*]");
}

QString MdiChild::strippedName(const QString &fullFileName)
{
	return QFileInfo(fullFileName).fileName();
}

#include "mdichild.moc"
