/***************************************************************************
 *   Copyright (C) 2007 by Andrea Moro   *
 *   andrea_moro@email.it   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 ***************************************************************************/
#include <QtGui>
#include "playfair.h"
#include "mdichild.h"
#include "mainWindow.h"

Playfair::Playfair(mainWindow *main,QString text, QString title)
{
	setupUi(this);
	workText=text;
	workTitle=title;
	mw=main;
	int i,j,posx,posy=193;
	for(i=0;i<6;i++)
	{
		posx=40;
		for(j=0;j<6;j++)
		{
			cell[i][j]= new QLineEdit(this);
			cell[i][j]->setEnabled(false);
			cell[i][j]->setGeometry(posx,posy,20,25);
			posx+=26;
		}
		posy+=31;
	}
	createConnections();
	keyLine->setFocus(Qt::OtherFocusReason);
	upMatr();
}
void Playfair::upMatr()
{
	if(fiveRadio->isChecked())
	{
		QRegExp re("[a-ik-zA-IK-Z]*");
		QRegExpValidator *v = new QRegExpValidator(re,this);
		keyLine->setValidator(v);
		int i;
		QString newKey;
		for(i=0;i<keyLine->text().length();i++)
			if(keyLine->text()[i]!=QChar('J'))
				newKey.append(keyLine->text()[i]);
		keyLine->setText(newKey);
		cell[0][0]->setText("A");
		cell[0][1]->setText("B");
		cell[0][2]->setText("C");
		cell[0][3]->setText("D");
		cell[0][4]->setText("E");
		cell[0][5]->setText("");
		cell[1][0]->setText("F");
		cell[1][1]->setText("G");
		cell[1][2]->setText("H");
		cell[1][3]->setText("I");
		cell[1][4]->setText("K");
		cell[1][5]->setText("");
		cell[2][0]->setText("V");
		cell[2][1]->setText("W");
		cell[2][2]->setText("X");
		cell[2][3]->setText("Y");
		cell[2][4]->setText("Z");
		cell[2][5]->setText("");
		cell[3][0]->setText("L");
		cell[3][1]->setText("M");
		cell[3][2]->setText("N");
		cell[3][3]->setText("O");
		cell[3][4]->setText("P");
		cell[3][5]->setText("");
		cell[4][0]->setText("Q");
		cell[4][1]->setText("R");
		cell[4][2]->setText("S");
		cell[4][3]->setText("T");
		cell[4][4]->setText("U");
		cell[4][5]->setText("");
		cell[5][0]->setText("");
		cell[5][1]->setText("");
		cell[5][2]->setText("");
		cell[5][3]->setText("");
		cell[5][4]->setText("");
		cell[5][5]->setText("");
	}
	else
	{
		QRegExp re("[a-zA-Z]*");
		QRegExpValidator *v = new QRegExpValidator(re,this);
		keyLine->setValidator(v);
		cell[0][0]->setText("A");
		cell[0][1]->setText("B");
		cell[0][2]->setText("C");
		cell[0][3]->setText("D");
		cell[0][4]->setText("E");
		cell[0][5]->setText("F");
		cell[1][0]->setText("G");
		cell[1][1]->setText("H");
		cell[1][2]->setText("I");
		cell[1][3]->setText("J");
		cell[1][4]->setText("K");
		cell[1][5]->setText("L");
		cell[2][0]->setText("Y");
		cell[2][1]->setText("Z");
		cell[2][2]->setText("0");
		cell[2][3]->setText("1");
		cell[2][4]->setText("2");
		cell[2][5]->setText("3");
		cell[3][0]->setText("M");
		cell[3][1]->setText("N");
		cell[3][2]->setText("O");
		cell[3][3]->setText("P");
		cell[3][4]->setText("Q");
		cell[3][5]->setText("R");
		cell[4][0]->setText("S");
		cell[4][1]->setText("T");
		cell[4][2]->setText("U");
		cell[4][3]->setText("V");
		cell[4][4]->setText("W");
		cell[4][5]->setText("X");
		cell[5][0]->setText("4");
		cell[5][1]->setText("5");
		cell[5][2]->setText("6");
		cell[5][3]->setText("7");
		cell[5][4]->setText("8");
		cell[5][5]->setText("9");
	}
	upKey();
}
void Playfair::upKey()
{
	QString key,rem;
	int i,k,mLen=5,z,j;
	keyLine->setText(keyLine->text().toUpper());
	if(sixRadio->isChecked())
		mLen=6;
	for(i=0;i<keyLine->text().length();i++)
	{
		bool p=false;
		for(k=0;k<key.length();k++)
			if(key[k]==keyLine->text()[i])
				p=true;
		if(!p)
			key.append(keyLine->text()[i]);
	}
	z=0;
	for(i=0;i<mLen;i++)
		for(j=0;j<mLen;j++)
		{
			if(z>=key.length())
				return;
			bool p=false;
			for(k=0;k<key.length();k++)
				if(cell[i][j]->text()[0]==key[k])
					p=true;
			if(!p)
				rem.append(cell[i][j]->text()[0]);
			cell[i][j]->setText(QChar(key[z++]));
			if(z>=key.length())
			{
				key.append(rem);
				rem.clear();
			}
		}
}
void Playfair::createConnections()
{
	connect(keyLine,SIGNAL(textEdited(const QString&)),this,SLOT(upMatr()));
	connect(fiveRadio,SIGNAL(clicked()),this,SLOT(upMatr()));
	connect(sixRadio,SIGNAL(clicked()),this,SLOT(upMatr()));
	connect(encryptButton,SIGNAL(clicked()),this,SLOT(encrypt()));
	connect(decryptButton,SIGNAL(clicked()),this,SLOT(decrypt()));
}
void Playfair::decrypt()
{
	QString title;
	title.append(tr("Playfair decryption of <"));
	title.append(workTitle);
	title.append(tr(">, key <"));
	title.append(keyLine->text());
	title.append(">");
	QString pair,noCrypt[3],cryptText;
	int len=workText.length(),i,k,j,kLen=5;
	if(sixRadio->isChecked())
		kLen=6;
	for(i=0;i<len;i++)
	{
		if(isalpha(workText.toAscii()[i])&&(fiveRadio->isChecked()))
			pair.append(workText[i]);
		else if (isalnum(workText.toAscii()[i])&&(sixRadio->isChecked()))
			pair.append(workText[i]);
		else
			noCrypt[pair.length()].append(workText[i]);
		if((pair.length()<2)&&(i!=len-1))
			continue;
		int posx1=0,posy1=0,posx2=0,posy2=0,ch1x,ch1y,ch2x,ch2y;
		for(j=0;j<kLen;j++)
			for(k=0;k<kLen;k++)
				if(cell[j][k]->text()[0]==pair.toUpper()[0])
				{
				posx1=j;
				posy1=k;
				}
		for(j=0;j<kLen;j++)
			for(k=0;k<kLen;k++)
				if(cell[j][k]->text()[0]==pair.toUpper()[1])
				{
				posx2=j;
				posy2=k;
				}
		if((posy1==posy2)&&(posx1==posx2))
		{
			ch1x=ch2x=posx1;
			ch1y=ch2y=(posy1-1+kLen)%kLen;
		}
		else if(posy1==posy2)
		{
			ch2y=ch1y=posy1;
			ch1x=(posx1+1)%kLen;
			ch2x=(posx2+1)%kLen;
		}
		else if((posx1==posx2)||(pair.length()==1))
		{
			ch2x=ch1x=posx1;
			ch1y=(posy1+1)%kLen;
			ch2y=(posy2+1)%kLen;
		}
		else
		{
			ch1x=posx1;
			ch1y=posy2;
			ch2x=posx2;
			ch2y=posy1;
		}
		cryptText.append(noCrypt[0]);
		if(pair[0].isUpper())
			cryptText.append(cell[ch1x][ch1y]->text());
		else
			cryptText.append(cell[ch1x][ch1y]->text().toLower());
		cryptText.append(noCrypt[1]);
		if(pair.length()!=1)
		{
			if(pair[1].isUpper())
				cryptText.append(cell[ch2x][ch2y]->text());
			else
				cryptText.append(cell[ch2x][ch2y]->text().toLower());
			cryptText.append(noCrypt[2]);
		}
		for(j=0;j<3;j++)
			noCrypt[j].clear();
		pair.clear();
	}
	mw->newSubWindow(title,cryptText);
	close();
}
void Playfair::encrypt()
{
	QString title;
	title.append(tr("Playfair encryption of <"));
	title.append(workTitle);
	title.append(tr(">, key <"));
	title.append(keyLine->text());
	title.append(">");
	QString pair,noCrypt[3],cryptText;
	int len=workText.length(),i,k,j,kLen=5;
	if(sixRadio->isChecked())
		kLen=6;
	for(i=0;i<len;i++)
	{
		if(isalpha(workText.toAscii()[i])&&(fiveRadio->isChecked()))
			pair.append(workText[i]);
		else if (isalnum(workText.toAscii()[i])&&(sixRadio->isChecked()))
			pair.append(workText[i]);
		else
			noCrypt[pair.length()].append(workText[i]);
		if((pair.length()<2)&&(i!=len-1))
			continue;
		int posx1=0,posy1=0,posx2=0,posy2=0,ch1x,ch1y,ch2x,ch2y;
		for(j=0;j<kLen;j++)
			for(k=0;k<kLen;k++)
				if(cell[j][k]->text()[0]==pair.toUpper()[0])
				{
					posx1=j;
					posy1=k;
				}
		for(j=0;j<kLen;j++)
			for(k=0;k<kLen;k++)
				if(cell[j][k]->text()[0]==pair.toUpper()[1])
				{
					posx2=j;
					posy2=k;
				}
		if((posy1==posy2)&&(posx1==posx2))
		{
			ch1x=ch2x=posx1;
			ch1y=ch2y=(posy1+1)%kLen;
		}
		else if(posy1==posy2)
		{
			ch2y=ch1y=posy1;
			ch1x=(posx1-1+kLen)%kLen;
			ch2x=(posx2-1+kLen)%kLen;
		}
		else if((posx1==posx2)||(pair.length()==1))
		{
			ch2x=ch1x=posx1;
			ch1y=(posy1-1+kLen)%kLen;
			ch2y=(posy2-1+kLen)%kLen;
		}
		else
		{
			ch1x=posx1;
			ch1y=posy2;
			ch2x=posx2;
			ch2y=posy1;
		}
		cryptText.append(noCrypt[0]);
		if(pair.length()>0)
		{
			if(pair[0].isLower()&&mw->options.text_opt.keep_case)
				cryptText.append(cell[ch1x][ch1y]->text().toLower());
			else
				cryptText.append(cell[ch1x][ch1y]->text());
			cryptText.append(noCrypt[1]);
		}
		if(pair.length()==2)
		{
			if(pair[1].isLower()&&mw->options.text_opt.keep_case)
				cryptText.append(cell[ch2x][ch2y]->text().toLower());
			else
				cryptText.append(cell[ch2x][ch2y]->text());
			cryptText.append(noCrypt[2]);
		}
		for(j=0;j<3;j++)
			noCrypt[j].clear();
		pair.clear();
	}
	mw->newSubWindow(title,cryptText);
	close();
}

#include "playfair.moc"
