/***************************************************************************
 *   Copyright (C) 2007 by Andrea Moro   *
 *   andrea_moro@email.it   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 ***************************************************************************/
#ifndef PLAYFAIR_H
#define PLAYFAIR_H

#include "ui_playfair.h"

class mainWindow;

/** 
 * @short User interface and algorithms for Playfair cipher.
 *
 * The Playfair cipher. This class provides the cryptographic algorithms
 * and user interface handling.
 */
class Playfair :
		public QWidget,
		public Ui::Playfair
{
	Q_OBJECT
public:
	/**
	 * Creates a new Playfair instance.
	 *
	 * @param *main the main window of CrypTooLinux
	 * @param text the text to encrypt or decrypt
	 * @param title the title of the window taht contain the text to encrypt or decrypt
	 */
	Playfair(mainWindow *main,QString text, QString title);
private slots:
	/**
	 * Check the key and update the matrix to the new key.
	 */
	void upKey();
	/**
	 * Update the dimensions of matrix and call upKey.
	 */
	void upMatr();
	/**
	 * Decrypt the workText with the key and matrix inserted, close the Playfair window
	 * and create a new window in CrypTooLinux main window with the decrypted text.
	 */
	void decrypt();
	/**
	 * Encrypt the workText with the key and matrix inserted, close the Playfair window
	 * and create a new window in CrypTooLinux main window with the encrypted text.
	 */
	void encrypt();
private:
	/**
	 * Makes the connections between signal of the ui and the slot functions.
	 */
	void createConnections();
	/**
	 * The text to encrypt or decrypt.
	 */
	QString workText;
	/**
	 * The title of window in CrypTooLinux main window that contain the text
	 * to encrypt or decrypt.
	 */
	QString workTitle;
	/**
	 * CrypTooLinux main window.
	 */
	mainWindow *mw;
	/**
	 * The key matrix in the ui.
	 */
	QLineEdit *cell[6][6];
};
#endif
