/***************************************************************************
 *   Copyright (C) 2007 by Andrea Moro   *
 *   andrea_moro@email.it   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 ***************************************************************************/

#include <QtGui>
#include "textOpt.h"
#include "mainWindow.h"

textOpt::textOpt(mainWindow *main)
{
	setupUi(this);
	connect(applyButton,SIGNAL(clicked()),this,SLOT(apply()));
	connect(findButton,SIGNAL(clicked()),this,SLOT(findFile()));
	connect(alphabet,SIGNAL(textChanged(const QString &)),this,SLOT(updLabel()));
	connect(uppercase,SIGNAL(stateChanged(int)),this,SLOT(updUpper()));
	connect(lowercase,SIGNAL(stateChanged(int)),this,SLOT(updLower()));
	connect(punctuation,SIGNAL(stateChanged(int)),this,SLOT(updPunct()));
	connect(space,SIGNAL(stateChanged(int)),this,SLOT(updSpace()));
	connect(numerals,SIGNAL(stateChanged(int)),this,SLOT(updNumbers()));
	connect(distinguish_case,SIGNAL(stateChanged(int)),this,SLOT(checkCase()));
	connect(keep_case,SIGNAL(stateChanged(int)),this,SLOT(checkCase()));
	connect(padChar,SIGNAL(textChanged(const QString &)),this,SLOT(checkPad()));

	mw=main;

	char_unchanged->setCheckState(mw->options.text_opt.char_unchanged?(Qt::Checked):(Qt::Unchecked));
	keep_case->setCheckState(mw->options.text_opt.keep_case?(Qt::Checked):(Qt::Unchecked));
	distinguish_case->setCheckState(mw->options.text_opt.distinguish_case?(Qt::Checked):(Qt::Unchecked));
	alphabet->setText(mw->options.text_opt.alphabet);
	stat_file->setText(mw->options.text_opt.stat_file);
	separate->setCheckState(mw->options.text_opt.separate?(Qt::Checked):(Qt::Unchecked));
	separator->setText(mw->options.text_opt.separator);
	aEqual1->setChecked(mw->options.text_opt.aEqual1);
	aEqual0->setChecked(!mw->options.text_opt.aEqual1);
	newPadChar->setCheckState(mw->options.text_opt.padChar!='\0'?(Qt::Checked):(Qt::Unchecked));
	padChar->setText(mw->options.text_opt.padChar);
}

void textOpt::checkPad()
{
	if(alphabet->text().indexOf(padChar->text())==-1)
		padChar->setText("");
}

void textOpt::checkCase()
{
	QCheckBox *tmp=qobject_cast<QCheckBox*>(sender());
	if((tmp==distinguish_case)&&(!distinguish_case->checkState()))
	{
		alphabet->setText(alphabet->text().toUpper());
		int i,j;
		for(i=0;i<alphabet->text().length();i++)
			if((j=alphabet->text().indexOf(alphabet->text()[i],i+1))!=-1)
				alphabet->setText(alphabet->text().remove(j,1));
	}
	if(!tmp->checkState())
		return;
	if(distinguish_case->checkState() && keep_case->checkState())
	{
		if(tmp==distinguish_case)
			keep_case->setCheckState(Qt::Unchecked);
		else
		{
			keep_case->setCheckState(Qt::Unchecked);
			QMessageBox::warning(this, tr("Warning"),
				tr("To choose this option, you must first deactivate the option"
				"below\n\"Distinguish between uppercase and lowercase\"!"));
		}
	}
}

void textOpt::updUpper()
{
	char a[27]="ABCDEFGHIJKLMNOPQRSTUVWXYZ";
	int i;
	if(uppercase->checkState()==Qt::Checked)
	{
		for(i=0;i<26;i++)
			if(alphabet->text().indexOf(a[i])==-1)
				alphabet->setText(alphabet->text().append(a[i]));
	}
	else if(uppercase->checkState()==Qt::Unchecked)
		alphabet->setText(alphabet->text().remove(QRegExp("[A-Z].")));
}
void textOpt::updLower()
{
	char a[27]="abcdefghijklmnopqrstuvwxyz";
	int i;
	if(lowercase->checkState()==Qt::Checked)
	{
		for(i=0;i<26;i++)
			if(alphabet->text().indexOf(a[i])==-1)
				alphabet->setText(alphabet->text().append(a[i]));
	}
	else if(lowercase->checkState()==Qt::Unchecked)
		alphabet->setText(alphabet->text().remove(QRegExp("[a-z].")));
}
void textOpt::updPunct()
{
	char a[9]=".,:;!?()";
	int i;
	if(punctuation->checkState()==Qt::Checked)
	{
		for(i=0;i<8;i++)
			if(alphabet->text().indexOf(a[i])==-1)
				alphabet->setText(alphabet->text().append(a[i]));
	}
	else if(punctuation->checkState()==Qt::Unchecked)
		alphabet->setText(alphabet->text().remove(QRegExp("[.,:;!?()].")));
}
void textOpt::updSpace()
{
	char a=' ';
	if(space->checkState()==Qt::Checked)
	{
		if(alphabet->text().indexOf(a)==-1)
			alphabet->setText(alphabet->text().append(a));
	}
	else if(space->checkState()==Qt::Unchecked)
		alphabet->setText(alphabet->text().remove(' '));
}
void textOpt::updNumbers()
{
	char a[11]="0123456789";
	int i;
	if(numerals->checkState()==Qt::Checked)
	{
		for(i=0;i<10;i++)
			if(alphabet->text().indexOf(a[i])==-1)
				alphabet->setText(alphabet->text().append(a[i]));
	}
	else if(numerals->checkState()==Qt::Unchecked)
		alphabet->setText(alphabet->text().remove(QRegExp("[0-9].")));
}

void textOpt::updLabel()
{
	int i;
	int pos=alphabet->cursorPosition();
	if(pos!=0)
		pos--;
	QString text=alphabet->text();
	QChar n=text[pos];
	for(i=0;i<text.length();i++)
		if(i!=pos)
			if(text[i]==n)
			{
				text.remove(pos,1);
				alphabet->setText(text);
				alphabet->setCursorPosition(pos);
				break;
			}	
	char tmp[10];
	sprintf(tmp,"%d",alphabet->text().length());
	alphabetLabel->setText(tr("Alphabet to use (")+
			tmp+tr(" characters):"));

	char up[27]="ABCDEFGHIJKLMNOPQRSTUVWXYZ";
	char low[27]="abcdefghijklmnopqrstuvwxyz";
	char pun[9]=".,:;!?()";
	char sp=' ';
	char num[11]="0123456789";
	int found=0;

	for(i=0;i<26;i++)
		if(alphabet->text().indexOf(up[i])!=-1)
			found++;
	if(found==0)
		uppercase->setCheckState(Qt::Unchecked);
	else if(found==26)
		uppercase->setCheckState(Qt::Checked);
	else
		uppercase->setCheckState(Qt::PartiallyChecked);

	found=0;
	for(i=0;i<26;i++)
		if(alphabet->text().indexOf(low[i])!=-1)
			found++;
	if(found==0)
		lowercase->setCheckState(Qt::Unchecked);
	else if(found==26)
	{
		lowercase->setCheckState(Qt::Checked);
		keep_case->setCheckState(Qt::Unchecked);
		distinguish_case->setCheckState(Qt::Checked);
	}
	else
	{
		lowercase->setCheckState(Qt::PartiallyChecked);
		keep_case->setCheckState(Qt::Unchecked);
		distinguish_case->setCheckState(Qt::Checked);
	}

	found=0;
	for(i=0;i<8;i++)
		if(alphabet->text().indexOf(pun[i])!=-1)
			found++;
	if(found==0)
		punctuation->setCheckState(Qt::Unchecked);
	else if(found==8)
		punctuation->setCheckState(Qt::Checked);
	else
		punctuation->setCheckState(Qt::PartiallyChecked);
	
	if(alphabet->text().indexOf(sp)==-1)
		space->setCheckState(Qt::Unchecked);
	else
		space->setCheckState(Qt::Checked);

	found=0;
	for(i=0;i<10;i++)
		if(alphabet->text().indexOf(num[i])!=-1)
			found++;
	if(found==0)
		numerals->setCheckState(Qt::Unchecked);
	else if(found==10)
		numerals->setCheckState(Qt::Checked);
	else
		numerals->setCheckState(Qt::PartiallyChecked);
	checkPad();
}

void textOpt::findFile()
{
	QString fileName = QFileDialog::getOpenFileName(this),key,title;
	stat_file->setText(fileName);
}

void textOpt::apply()
{
	if((stat_file->text()!="")&&(!QFile::exists(stat_file->text())))
	{
		QMessageBox::warning(this, tr("Warning"),
				     tr("The stat file doesn't exists!"));
		return;
	}
	if(separate->checkState()==Qt::Checked)
		if(separator->text().length()!=1)
		{
			QMessageBox::warning(this, tr("Warning"),
					     tr("The separator must be a single character!"));
			return;
		}
	mw->options.text_opt.alphabet=alphabet->text();
	mw->options.text_opt.char_unchanged=char_unchanged->checkState()==Qt::Checked?true:false;
	mw->options.text_opt.keep_case=keep_case->checkState()==Qt::Checked?true:false;
	mw->options.text_opt.distinguish_case=distinguish_case->checkState()==Qt::Checked?true:false;
	mw->options.text_opt.stat_file=stat_file->text();
	mw->options.text_opt.separate=separate->checkState()==Qt::Checked?true:false;
	mw->options.text_opt.aEqual1=aEqual1->isChecked();
	if(mw->options.text_opt.separate)
		mw->options.text_opt.separator=separator->text()[0];
	mw->opttext_changed->activate(QAction::Trigger);
	mw->options.text_opt.padChar='\0';
	if(newPadChar->isChecked())
		mw->options.text_opt.padChar=QChar(padChar->text()[0]);
	close();
}

#include "textOpt.moc"
