/***************************************************************************
 *   Copyright (C) 2007 by Andrea Moro   *
 *   andrea_moro@email.it   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 ***************************************************************************/

#ifndef TEXTOPT_H
#define TEXTOPT_H

#include "ui_text_opt.h"

class mainWindow;

/** 
 * @short User interface for Text Options.
 *
 * Window to edit options related to text, like alphabet, character to use
 * like separator.
 */
class textOpt :
		public QWidget,
		public Ui::textOpt
{
	Q_OBJECT
public:
	/**
	 * Creates a new textOpt instance.
	 *
	 * @param *main the main window of CrypTooLinux
	 */
	textOpt(mainWindow *main);
private slots:
	/**
	 * Save the options in the struct of the options in the main window.
	 */
	void apply();
	/**
	 * Open a QFileDialog and set stat_file to the file selected.
	 */
	void findFile();
	/**
	 * Check the change in the alphabet and update the associate checks.
	 */
	void updLabel();
	/**
	 * Update the alphabet.
	 */
	void updUpper();
	/**
	 * Update the alphabet.
	 */
	void updLower();
	/**
	 * Update the alphabet.
	 */
	void updPunct();
	/**
	 * Update the alphabet.
	 */
	void updSpace();
	/**
	 * Update the alphabet.
	 */
	void updNumbers();
	/**
	 * Check that the case options cannot be, in the same time, checked.
	 */
	void checkCase();
	/**
	 * Check if the pad character is in the alphabet.
	 */
	void checkPad();
private:
	/**
	 * CrypTooLinux main window.
	 */
	mainWindow *mw;
};
#endif
