/***************************************************************************
 *   Copyright (C) 2007 by Andrea Moro   *
 *   andrea_moro@email.it   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 ***************************************************************************/

#include <QtGui>
#include "vigenere.h"
#include "mdichild.h"
#include "mainWindow.h"

Vigenere::Vigenere(mainWindow *main,QString text, QString title)
{
	setupUi(this);
	workText=text;
	workTitle=title;
	mw=main;
	createConnections();
	initializeDialog();
}
void Vigenere::initializeDialog()
{
	QRegExp re;
	if(mw->options.text_opt.keep_case)
		re=QRegExp("["+mw->options.text_opt.alphabet+mw->options.text_opt.alphabet.toLower()+"]*");
	else
		re=QRegExp("["+mw->options.text_opt.alphabet+"]*");
	QRegExpValidator *v = new QRegExpValidator(re,this);
	keyEdit->setValidator(v);
}
void Vigenere::createConnections()
{
	connect(keyEdit,SIGNAL(textEdited(const QString &)),this,SLOT(activeButton()));
	connect(encryptButton,SIGNAL(clicked()),this,SLOT(encrypt()));
	connect(decryptButton,SIGNAL(clicked()),this,SLOT(decrypt()));
	connect(textOptButton,SIGNAL(clicked()),mw,SLOT(opt_text()));
	connect(mw->opttext_changed,SIGNAL(triggered()),this,SLOT(initializeDialog()));
}
void Vigenere::activeButton()
{
	if(keyEdit->text().length()==0)
	{
		encryptButton->setEnabled(false);
		decryptButton->setEnabled(false);
	}
	else
	{
		encryptButton->setEnabled(true);
		decryptButton->setEnabled(true);
		if(mw->options.text_opt.keep_case)
			keyEdit->setText(keyEdit->text().toUpper());
	}
}
void Vigenere::decrypt()
{
	QString title;
	title.append(tr("Vigenère decryption of <"));
	title.append(workTitle);
	title.append(tr(">, key <"));
	title.append(keyEdit->text());
	title.append(">");
	QString alph=mw->options.text_opt.alphabet,newWorkText;
	int len=workText.length(),k=0,kLen=keyEdit->text().length(),i,alphLen=alph.length();
	int keyShift[len],pos;
	for(i=0;i<kLen;i++)
		keyShift[i]=alph.indexOf(keyEdit->text()[i]);
	for(i=0;i<len;i++)
	{
		Qt::CaseSensitivity cs;
		if(mw->options.text_opt.keep_case)
			cs = Qt::CaseInsensitive;
		else
			cs = Qt::CaseSensitive;

		if((pos=alph.indexOf(QChar(workText.toAscii()[i]),0,cs))!=-1)
		{
			pos=(pos-keyShift[k])%alphLen;
			if(pos<0)
				pos=pos+alphLen;
			newWorkText.append(workText[i].isLower()&&mw->options.text_opt.keep_case?
				alph[pos].toLower():alph[pos]);
			k=(k+1)%kLen;
		}
		else if(mw->options.text_opt.char_unchanged)
			newWorkText.append(workText[i]);
	}
	mw->newSubWindow(title,newWorkText);
	close();
}
void Vigenere::encrypt()
{
	QString title;
	title.append(tr("Vigenère encryption of <"));
	title.append(workTitle);
	title.append(tr(">, key <"));
	title.append(keyEdit->text());
	title.append(">");

	QString alph=mw->options.text_opt.alphabet,newWorkText;
	int len=workText.length(),k=0,kLen=keyEdit->text().length(),i,alphLen=alph.length();
	int keyShift[len],pos;
	for(i=0;i<kLen;i++)
		keyShift[i]=alph.indexOf(keyEdit->text()[i]);
	for(i=0;i<len;i++)
	{
		Qt::CaseSensitivity cs;
		if(mw->options.text_opt.keep_case)
			cs = Qt::CaseInsensitive;
		else
			cs = Qt::CaseSensitive;

		if((pos=alph.indexOf(QChar(workText.toAscii()[i]),0,cs))!=-1)
		{
			newWorkText.append(workText[i].isLower()&&mw->options.text_opt.keep_case?
				alph[(pos+keyShift[k])%alphLen].toLower():alph[(pos+keyShift[k])%alphLen]);
			k=(k+1)%kLen;
		}
		else if(mw->options.text_opt.char_unchanged)
			newWorkText.append(workText[i]);
	}
	mw->newSubWindow(title,newWorkText);
	close();
}

#include "vigenere.moc"
